//go:generate go mod vendor
//go:generate protoc --go_out=. --go_opt=paths=source_relative --go-grpc_out=. --go-grpc_opt=paths=source_relative --proto_path=. --proto_path=vendor/gitlab.mashplace.com/mashplace/backend mashroom_offer.proto
//go:generate rm -r vendor
//go:generate mockgen -package=mockoffer -destination=mock/client.go . OfferServiceClient
//go:generate go fmt ./...
//go:generate go fix ./...
//go:generate goimports -w ./
//go:generate go mod tidy

// Package offer - api definition
package offer // import "gitlab.mashplace.com/mashplace/backend/offer/api"

import (
	_ "gitlab.mashplace.com/mashplace/backend/common/api/generic"       // required vendor
	_ "gitlab.mashplace.com/mashplace/backend/common/api/generic/query" // required vendor
	_ "gitlab.mashplace.com/mashplace/backend/extras/api"               // required vendor
	invoice "gitlab.mashplace.com/mashplace/backend/invoice/api"

	_ "github.com/golang/mock/mockgen/model"
)

// Percent for a rent plan
func (x RentPlan) Percent() float64 {
	switch x {
	case RentPlan_RENT_PLAN_ESSENTIAL:
		return 2
	case RentPlan_RENT_PLAN_SECURED:
		return 5
	}

	return 0
}

// DataPeriodicPaymentFrequency to get invoice payment frequency enum for not rto offer
var DataPeriodicPaymentFrequency = map[int32]invoice.PeriodicPaymentFrequency{
	1:  invoice.PeriodicPaymentFrequency_FREQUENCY_MONTHLY,
	3:  invoice.PeriodicPaymentFrequency_FREQUENCY_QUARTERLY,
	6:  invoice.PeriodicPaymentFrequency_FREQUENCY_TWICE_A_YEAR,
	12: invoice.PeriodicPaymentFrequency_FREQUENCY_YEARLY,
}

// Offer max and min rent payment days
const (
	// FirstRentPaymentDay is the min value of rent payment day
	FirstRentPaymentDay int = 1
	// LastRentPaymentDay is the max value of rent payment day
	LastRentPaymentDay int = 28
)

// PaymentDayClamp is a handle rent payment day to avoid unsuitable values
func PaymentDayClamp(d int) (day int) {
	day = d

	if day < FirstRentPaymentDay {
		day = FirstRentPaymentDay
	}

	if day > LastRentPaymentDay {
		day = LastRentPaymentDay
	}

	return
}
