package impl

import (
	"context"
	"time"

	"gitlab.mashplace.com/mashplace/backend/common/api/grpcctx"

	"gitlab.mashplace.com/mashplace/backend/offer/internal/connections"
	"gitlab.mashplace.com/mashplace/backend/offer/internal/schedule/hourly"
	"gitlab.mashplace.com/mashplace/backend/offer/internal/schedule/oneoff"
	schedule "gitlab.mashplace.com/mashplace/backend/schedule/api"
	"google.golang.org/protobuf/types/known/durationpb"
)

const (
	scheduleHandlerHourly = "hourly"
)

func (server *serverImpl) RegisterSchedules(ctx context.Context) error {
	_, err := connections.ScheduleService.ReminderSetup(ctx, &schedule.Reminder_SetupRequest{
		Origin: "offer",
		Setups: map[string]*schedule.Reminder_Setup{
			scheduleHandlerHourly: {
				ConsumerUrl: connections.SelfRemote,
				Period:      durationpb.New(time.Hour),
				Reminder: &schedule.Reminder{
					Data: nil,
				},
			},
		},
	})

	return err
}

func (server *serverImpl) ConsumeReminder(
	ctx context.Context,
	req *schedule.Reminder_ConsumeRequest,
) (resp *schedule.Reminder_ConsumeResponse, err error) {
	bgctx := grpcctx.ValuesBackgroundContext(ctx)

	switch req.GetTaskId() {
	case "":
		go oneoff.ConsumeReminderInvocation(bgctx, req)
	case scheduleHandlerHourly:
		go hourly.ConsumeReminderHourly(bgctx, req)
	}

	return &schedule.Reminder_ConsumeResponse{}, nil
}
