package rto

import (
	"time"

	api "gitlab.mashplace.com/mashplace/backend/offer/api/rto"
)

const (
	CloseComConfidence         = 100
	CloseComValueCurrencyGBP   = "GBP"
	CloseComValuePeriodOneTime = "one_time"
	CloseComNoteRto            = ""
)

type CloseComOpportunity interface {
	SetLeadID(leadID string) CloseComOpportunity
}

type CloseComOpportunityDev struct {
	Termination    string `json:"custom.cf_CnBrm7rWWBWGKomkoZzIXopHQ3ZIHZxV8HjfdTtw9Fx,omitempty"`
	MoveIn         string `json:"custom.cf_LNUC6cdInPD2yTNd4uIdvoOafiSiQbUFOQADAAJSQJ0,omitempty"`
	Property       string `json:"custom.cf_HcJ5xwMay9aEGRbMPNIfqekYDlagmh3O2rdcuRbnhiR,omitempty"`
	StatusId       string `json:"status_id,omitempty"`
	Offer          string `json:"custom.cf_r8CE1QwEJ57OzryBmac65ptNwJXTBUCx9V0WWznZGYp,omitempty"`
	ValueCurrency  string `json:"value_currency,omitempty"`
	LeadID         string `json:"lead_id,omitempty"`
	Tenant         string `json:"custom.cf_tqJEk6V1yf56IbNsu8XdBkhBKRuYqvCwCVR6kgPx75Y,omitempty"`
	LastStatusDate string `json:"custom.cf_e4wR9Ny3ve4RAfAxZfXoVZM2Gn1cFAQ99PIpty1aGuK,omitempty"`
	LastStatus     string `json:"custom.cf_lRb3iyS0artQL8ABjyoVkOC0VRjLSx4SpF0a6dkjmE3,omitempty"`
	Address        string `json:"custom.cf_1N5R6c7Gd1emlzrIQ66LpgmsQo0puznAtpFnTx5jcmJ,omitempty"`
	Plan           string `json:"custom.cf_WxtJkJIwAWofSLATMCQma1PYWF009ud2AUh8umfSy8z,omitempty"`
	ValuePeriod    string `json:"value_period,omitempty"`
	Note           string `json:"note,omitempty"`
	PCM            int    `json:"custom.cf_wpWMsU3qbAeTjC8WHm7ANWQrR9IVMsaBIsSDD02Ziqh,omitempty"`
	Value          int    `json:"value,omitempty"`
	Confidence     int    `json:"confidence,omitempty"`
}

func (ccod *CloseComOpportunityDev) SetLeadID(leadID string) CloseComOpportunity {
	ccod.LeadID = leadID

	return ccod
}

type CloseComOpportunityProd struct {
	Termination    string `json:"custom.cf_WWtEu4A6WFuft2b82mOsy8VCY3LE3ejCTU6mw5q7oVU,omitempty"`
	MoveIn         string `json:"custom.cf_2419ycQEgiA4Bsc68w60ozKlkkWxeDAvBMrscZN8Uvt,omitempty"`
	Property       string `json:"custom.cf_BGVL3lMREZsSUnuMhnrkqpJJx6kyh2HDof8ZVjKdOVg,omitempty"`
	StatusId       string `json:"status_id,omitempty"`
	Offer          string `json:"custom.cf_ZT6Hx52N16hRhqntwfE9Rv9VGytK7VFOEoN0I751lUU,omitempty"`
	ValueCurrency  string `json:"value_currency,omitempty"`
	LeadID         string `json:"lead_id,omitempty"`
	Tenant         string `json:"custom.cf_FQQXRP4G69BGOYn3mUVM7orkd9VVWoa4YxKhbIFz8b7,omitempty"`
	LastStatusDate string `json:"custom.cf_0TNnzBLUT7pSxzWSiPembqHGGLuQnZTSl60htIksr8h,omitempty"`
	LastStatus     string `json:"custom.cf_IIJwX6THy285Rt2hTDt2D7np1R6NP2BTKxPpepixjog,omitempty"`
	Address        string `json:"custom.cf_G0xKkoRNGraKSQ9hRruImOF64MV4opGwEyURNqrqO4Q,omitempty"`
	Plan           string `json:"custom.cf_255z8nikOP21EyYMlYJCkuJoqsb8k1PsmWCP8FnXj6x,omitempty"`
	ValuePeriod    string `json:"value_period,omitempty"`
	Note           string `json:"note,omitempty"`
	PCM            int    `json:"custom.cf_9ZokIi3wYS0IFJ8EIBzm0rmoa9LJhxh6YJuuLPOdp3h,omitempty"`
	Value          int    `json:"value,omitempty"`
	Confidence     int    `json:"confidence,omitempty"`
}

func (ccop *CloseComOpportunityProd) SetLeadID(leadID string) CloseComOpportunity {
	ccop.LeadID = leadID

	return ccop
}

func NewCloseComOpportunityProd(rtoModel *Rto,
	rtoPreviousState api.RtoState, rtoPreviousStateDate time.Time) CloseComOpportunity {
	opt := &CloseComOpportunityProd{
		Value:         rtoModel.GetPrice(),
		ValuePeriod:   CloseComValuePeriodOneTime,
		ValueCurrency: CloseComValueCurrencyGBP,
		Note:          CloseComNoteRto,
		Confidence:    CloseComConfidence,
		StatusId:      GetProdCloseComStatusIDByRtoState(rtoModel.State),
		Address:       rtoModel.PropertyAddress,
		Property:      rtoModel.GetPropertyAdminLink(),
		Offer:         rtoModel.GetRtoAdminLink(),
		Tenant:        rtoModel.GetLeadTenantForCloseCom(),
		Plan:          rtoModel.GetRentPlanForCloseCom(),
		LastStatus:    GetCloseComStatusByRtoState(rtoPreviousState),
	}

	if rtoModel.GetPrice() != 0 {
		opt.PCM = rtoModel.GetPrice()
	}

	if !rtoModel.GetMoveIn().IsZero() {
		opt.MoveIn = GetDateForCloseCom(rtoModel.GetMoveIn())
	}

	if !rtoModel.GetEndDate().IsZero() {
		opt.Termination = GetDateForCloseCom(rtoModel.GetEndDate())
	}

	if !rtoPreviousStateDate.IsZero() {
		opt.LastStatusDate = GetDateForCloseCom(rtoPreviousStateDate)
	}

	return opt
}

func NewCloseComOpportunityDev(rtoModel *Rto,
	rtoPreviousState api.RtoState, rtoPreviousStateDate time.Time) CloseComOpportunity {
	opt := &CloseComOpportunityDev{
		Value:         rtoModel.GetPrice(),
		ValuePeriod:   CloseComValuePeriodOneTime,
		ValueCurrency: CloseComValueCurrencyGBP,
		Note:          CloseComNoteRto,
		Confidence:    CloseComConfidence,
		StatusId:      GetDevCloseComStatusIDByRtoState(rtoModel.State),
		Address:       rtoModel.PropertyAddress,
		Property:      rtoModel.GetPropertyAdminLink(),
		Offer:         rtoModel.GetRtoAdminLink(),
		Tenant:        rtoModel.GetLeadTenantForCloseCom(),
		Plan:          rtoModel.GetRentPlanForCloseCom(),
		LastStatus:    GetCloseComStatusByRtoState(rtoPreviousState),
	}

	if rtoModel.GetPrice() != 0 {
		opt.PCM = rtoModel.GetPrice()
	}

	if !rtoModel.GetMoveIn().IsZero() {
		opt.MoveIn = GetDateForCloseCom(rtoModel.GetMoveIn())
	}

	if !rtoModel.GetEndDate().IsZero() {
		opt.Termination = GetDateForCloseCom(rtoModel.GetEndDate())
	}

	if !rtoPreviousStateDate.IsZero() {
		opt.LastStatusDate = GetDateForCloseCom(rtoPreviousStateDate)
	}

	return opt
}

func GetDateForCloseCom(t time.Time) string {
	if t.IsZero() {
		t = time.Now()
	}

	return t.Format("2006-01-02")
}

func GetDevCloseComStatusIDByRtoState(rtoState api.RtoState) string {
	statusIDByRtoStateMap := map[api.RtoState]string{
		api.RtoState_RTO_STATE_PENDING_SETUP:                    "stat_nj7DIEoWtWzCwfNWUc8wU7ET5mjmpcDXer7W3wHtMAS",
		api.RtoState_RTO_STATE_WAITING_FOR_TENANT_CONFIRMATION:  "stat_36fvW7Dx7iJaX8bt6js2dIirmXKeb3iZ85hnzDAx5TI",
		api.RtoState_RTO_STATE_WAITING_FOR_STANDING_ORDER_SETUP: "stat_P9C1fs5AwSERJ5ngUrvwkreHXfQAEoAPIsRSM0alIVB",
		api.RtoState_RTO_STATE_WAITING_FOR_PAYMENT:              "stat_K8e3mQOT9dPMZpsXv5TgRcSYGl4IaDdjjAGzkYzx45C",
		api.RtoState_RTO_STATE_ACTIVE:                           "stat_T7DH3rch0lR7V43CzncybFOzcDfx0YhIWc56KGmWDum",
		api.RtoState_RTO_STATE_ACTIVE_ROLLING_TENANCY:           "stat_8BveyQ1A1CmJMrqr6zalBzKON7yqSWIk646cLFCzKG2",
		api.RtoState_RTO_STATE_ENDING:                           "stat_CIUVCrnZhQj22MnpaDqf2YdTZQEBm4aEzR2peVLm2tf",
		api.RtoState_RTO_STATE_FINISHED:                         "stat_JIEYXrdcotq72NXIaWUbD5g6dfQmNOaiB7NEOD1HAvk",
		api.RtoState_RTO_STATE_CANCELLED:                        "stat_JdR3hKThr16AlSzQ4rIa1k69bFhxJJjGqoZAKaLdzMQ",
	}

	return statusIDByRtoStateMap[rtoState]
}

func GetProdCloseComStatusIDByRtoState(rtoState api.RtoState) string {
	statusIDByRtoStateMap := map[api.RtoState]string{
		api.RtoState_RTO_STATE_PENDING_SETUP:                    "stat_HA6l8N4rDTNOZoUAPvUgQ5lb7ByR6BSi8nXIMnnytMS",
		api.RtoState_RTO_STATE_WAITING_FOR_TENANT_CONFIRMATION:  "stat_1ArpxZripY3cYpEWZ3POXtiDIzH4B1TL0iEGOx2E9YM",
		api.RtoState_RTO_STATE_WAITING_FOR_STANDING_ORDER_SETUP: "stat_Yr7IqMINBHr8aq9RDfqoULtlUih3X9KTAsSiWyNW4yi",
		api.RtoState_RTO_STATE_WAITING_FOR_PAYMENT:              "stat_NmChj9kka2h8VdrEQEAoMK23ysHCaPeVIm1hVIeuJxg",
		api.RtoState_RTO_STATE_ACTIVE:                           "stat_bMfvrPEdFlZAPLzxn6LSHT5gdoZiPCRH5DPA0m6wS9F",
		api.RtoState_RTO_STATE_ACTIVE_ROLLING_TENANCY:           "stat_HaZb2kt10IMn7CApCdsTuiB40dYltkcFQNWS8QmfC4k",
		api.RtoState_RTO_STATE_ENDING:                           "stat_jVUbyEKhoe8R5L385zEoBOVn4F3KovJb0DID3E9xors",
		api.RtoState_RTO_STATE_FINISHED:                         "stat_IbFPokwoyLlBuS9isCvfIGJAxJ8jZqutNQ15ybUPiI0",
		api.RtoState_RTO_STATE_CANCELLED:                        "stat_RQGbrHDtIZMgOrdNYRnVP7jFEORiGoHuKtHQ1tCwP84",
	}

	return statusIDByRtoStateMap[rtoState]
}

func GetCloseComStatusByRtoState(rtoState api.RtoState) string {
	statusByRtoStateMap := map[api.RtoState]string{
		api.RtoState_RTO_STATE_PENDING_SETUP:                    "Landlord Setup",
		api.RtoState_RTO_STATE_WAITING_FOR_TENANT_CONFIRMATION:  "Tenant Standing Order",
		api.RtoState_RTO_STATE_WAITING_FOR_STANDING_ORDER_SETUP: "Standing Order Setup",
		api.RtoState_RTO_STATE_WAITING_FOR_PAYMENT:              "Await Payment",
		api.RtoState_RTO_STATE_ACTIVE:                           "Active",
		api.RtoState_RTO_STATE_ACTIVE_ROLLING_TENANCY:           "Rolling Tenancy",
		api.RtoState_RTO_STATE_ENDING:                           "Await Termination",
		api.RtoState_RTO_STATE_FINISHED:                         "Finished",
		api.RtoState_RTO_STATE_CANCELLED:                        "Cancelled",
	}

	return statusByRtoStateMap[rtoState]
}
