package rto

import (
	"fmt"
	"os"

	account "gitlab.mashplace.com/mashplace/backend/account/api"
	api "gitlab.mashplace.com/mashplace/backend/offer/api/rto"
	"gitlab.mashplace.com/mashplace/backend/offer/model"
)

type Landlord struct {
	User           string `bson:"user"`
	FirstName      string `bson:"first_name"`
	LastName       string `bson:"last_name"`
	EmailPrimary   string `bson:"email_primary"`
	PhonePrimary   string `bson:"phone_primary"`
	BillingAddress string `bson:"billing_address"`
	LeadID         string `bson:"lead_id"`
}

func (l *Landlord) GetAdminLink() string {
	return fmt.Sprintf("https://%s/#/user/%s/view", os.Getenv("ADMIN_SITE_URL"), l.GetID())
}

func (l *Landlord) GetID() string {
	return l.User
}

func (l *Landlord) GetFullName() string {
	return l.FirstName + " " + l.LastName
}

func CreateLandlord(id string, account *account.Account) *Landlord {
	var landlord *Landlord
	if id != "" {
		landlord = &Landlord{
			User:           id,
			FirstName:      account.GetDetailsValue().GetFirstNameValue(),
			LastName:       account.GetDetailsValue().GetLastNameValue(),
			EmailPrimary:   account.GetContactValue().GetEmailPrimaryValue(),
			PhonePrimary:   account.GetContactValue().GetPhonePrimaryValue(),
			BillingAddress: model.GetBillingAddressFromLocation(account.GetLocationValue()),
		}
	}

	return landlord
}

func CreateApiLandlord(landlord *Landlord) *api.Landlord {
	if landlord != nil {
		return &api.Landlord{
			User:           landlord.User,
			FirstName:      landlord.FirstName,
			LastName:       landlord.LastName,
			EmailPrimary:   landlord.EmailPrimary,
			PhonePrimary:   landlord.PhonePrimary,
			BillingAddress: landlord.BillingAddress,
		}
	}

	return nil
}
