package mongo_test

import (
	"testing"
	"time"

	api "gitlab.mashplace.com/mashplace/backend/offer/api/rto"
	"gitlab.mashplace.com/mashplace/backend/offer/internal/app/domain/rto"
	"gitlab.mashplace.com/mashplace/backend/offer/test"

	. "github.com/smartystreets/goconvey/convey"
	"go.mongodb.org/mongo-driver/bson/primitive"
)

func TestRepository(t *testing.T) {
	if test.CanRunTests() {
		Convey("Repository", t, func() {
			So(repository, ShouldImplement, (*rto.Repository)(nil))

			err := repository.Drop(ctx)
			So(err, ShouldBeNil)

			landlordID := "shimkenttikterchertibarligi"
			id := primitive.NewObjectID()
			now := time.Now()
			model := &rto.Rto{
				ID:        id,
				State:     api.RtoState_RTO_STATE_ACTIVE,
				CreatedAt: now.Add(-24 * time.Hour),
				UpdatedAt: now,
				Landlord: &rto.Landlord{
					User:         landlordID,
					EmailPrimary: test.KorshilerDogs,
				},
				Conditions: &rto.Conditions{
					MoveInDate: now,
					Price:      test.Chmolgani,
				},
			}
			err = repository.Create(ctx, model)
			So(err, ShouldBeNil)

			res, err := repository.FindAllByLandlordID(ctx, &rto.AllByLandlordIDFilter{
				States:     []api.RtoState{api.RtoState_RTO_STATE_PENDING_SETUP},
				LandlordID: landlordID,
			})
			So(err, ShouldBeNil)
			So(res, ShouldBeEmpty)

			res, err = repository.FindAllByLandlordID(ctx, &rto.AllByLandlordIDFilter{
				States:     []api.RtoState{api.RtoState_RTO_STATE_PENDING_SETUP, model.State},
				LandlordID: landlordID,
			})
			So(err, ShouldBeNil)
			So(len(res), ShouldEqual, 1)
			So(res[0].ID, ShouldEqual, model.ID)

			res, err = repository.FindAllByLandlordID(ctx, &rto.AllByLandlordIDFilter{
				States:     []api.RtoState{model.State},
				LandlordID: "oristardinsheshelerishalavalareken",
			})
			So(err, ShouldBeNil)
			So(res, ShouldBeEmpty)

			res, err = repository.FindAllEmptyLeadTenantUsersByStates(ctx,
				[]api.RtoState{api.RtoState_RTO_STATE_ACTIVE})
			So(err, ShouldBeNil)
			So(res, ShouldBeEmpty)

			firstName, lastName, email := "korshilerchmoriki", "mambetichmoriki", "aset-kulyash@suki.com"
			model.LeadTenant = &rto.Tenant{
				FirstName:    firstName,
				LastName:     lastName,
				EmailPrimary: email,
			}
			err = repository.MustUpdate(ctx, model)
			So(err, ShouldBeNil)

			res, err = repository.FindAllEmptyLeadTenantUsersByStates(ctx,
				[]api.RtoState{api.RtoState_RTO_STATE_ACTIVE})
			So(err, ShouldBeNil)
			So(len(res), ShouldEqual, 1)
			So(res[0].ID, ShouldEqual, model.ID)

			res, err = repository.FindAllEmptyLeadTenantUsersByEmail(ctx, email)
			So(err, ShouldBeNil)
			So(len(res), ShouldEqual, 1)
			So(res[0].ID, ShouldEqual, model.ID)

			model.LeadTenant.User = "ruchmoryu"
			err = repository.MustUpdate(ctx, model)
			So(err, ShouldBeNil)

			res, err = repository.FindAllEmptyLeadTenantUsersByStates(ctx,
				[]api.RtoState{api.RtoState_RTO_STATE_ACTIVE})
			So(err, ShouldBeNil)
			So(res, ShouldBeEmpty)

			res, err = repository.FindAllEmptyLeadTenantUsersByEmail(ctx, email)
			So(err, ShouldBeNil)
			So(res, ShouldBeEmpty)

			model.LeadTenant.User = ""
			err = repository.MustUpdate(ctx, model)
			So(err, ShouldBeNil)

			res, err = repository.FindAllEmptyLeadTenantUsersByStates(ctx,
				[]api.RtoState{api.RtoState_RTO_STATE_ACTIVE})
			So(err, ShouldBeNil)
			So(len(res), ShouldEqual, 1)
			So(res[0].ID, ShouldEqual, model.ID)

			res, err = repository.FindAllEmptyLeadTenantUsersByEmail(ctx, email)
			So(err, ShouldBeNil)
			So(len(res), ShouldEqual, 1)
			So(res[0].ID, ShouldEqual, model.ID)

			res, err = repository.FindAllByFilter(ctx, &rto.Filter{
				States:         []api.RtoState{api.RtoState_RTO_STATE_PENDING_SETUP},
				DateFrom:       now,
				DateTo:         now,
				DateColumnName: rto.DateColumnNameCreatedAt,
			})
			So(err, ShouldBeNil)
			So(res, ShouldBeEmpty)

			res, err = repository.FindAllByFilter(ctx, &rto.Filter{
				States:         []api.RtoState{model.State},
				DateFrom:       now.Add(-25 * time.Hour),
				DateTo:         now.Add(-23 * time.Hour),
				DateColumnName: rto.DateColumnNameCreatedAt,
			})
			So(err, ShouldBeNil)
			So(len(res), ShouldEqual, 1)
			So(res[0].ID, ShouldEqual, model.ID)

			res, err = repository.FindAllByFilter(ctx, &rto.Filter{
				States:         []api.RtoState{api.RtoState_RTO_STATE_PENDING_SETUP},
				DateFrom:       now.Add(-25 * time.Hour),
				DateTo:         now.Add(-23 * time.Hour),
				DateColumnName: rto.DateColumnNameCreatedAt,
			})
			So(err, ShouldBeNil)
			So(res, ShouldBeEmpty)

			res, err = repository.FindAllByFilter(ctx, &rto.Filter{
				States:         []api.RtoState{model.State},
				DateFrom:       now.Add(-23 * time.Hour),
				DateTo:         now.Add(2 * time.Hour),
				DateColumnName: rto.DateColumnNameCreatedAt,
			})
			So(err, ShouldBeNil)
			So(res, ShouldBeEmpty)

			res, err = repository.FindAllByFilter(ctx, &rto.Filter{
				States:           []api.RtoState{model.State},
				NotificationType: rto.NotificationPendingSetupOneDay,
				DateFrom:         now.Add(-25 * time.Hour),
				DateTo:           now.Add(-23 * time.Hour),
				DateColumnName:   rto.DateColumnNameCreatedAt,
			})
			So(err, ShouldBeNil)
			So(len(res), ShouldEqual, 1)
			So(res[0].ID, ShouldEqual, model.ID)

			res, err = repository.FindAllByFilter(ctx, &rto.Filter{
				States:         []api.RtoState{model.State},
				DateFrom:       now.Add(-25 * time.Hour),
				DateTo:         now.Add(-23 * time.Hour),
				DateColumnName: rto.DateColumnNameCreatedAt,
			})
			So(err, ShouldBeNil)
			So(len(res), ShouldEqual, 1)
			So(res[0].ID, ShouldEqual, model.ID)

			if model.Notifications == nil {
				model.Notifications = rto.Notifications{
					rto.NotificationPendingSetupOneDay: now,
				}
			}
			err = repository.MustUpdate(ctx, model)
			So(err, ShouldBeNil)

			res, err = repository.FindAllByFilter(ctx, &rto.Filter{
				States:           []api.RtoState{model.State},
				NotificationType: rto.NotificationPendingSetupOneDay,
				DateFrom:         now.Add(-25 * time.Hour),
				DateTo:           now.Add(-23 * time.Hour),
				DateColumnName:   rto.DateColumnNameCreatedAt,
			})
			So(err, ShouldBeNil)
			So(res, ShouldBeEmpty)

			res, err = repository.FindAllByFilter(ctx, &rto.Filter{
				States:           []api.RtoState{model.State},
				NotificationType: rto.NotificationPendingSetupTwoDays,
				DateFrom:         now.Add(-25 * time.Hour),
				DateTo:           now.Add(-23 * time.Hour),
				DateColumnName:   rto.DateColumnNameCreatedAt,
			})
			So(err, ShouldBeNil)
			So(len(res), ShouldEqual, 1)
			So(res[0].ID, ShouldEqual, model.ID)
			So(int(now.Sub(res[0].GetDate(rto.DateColumnNameCreatedAt)).Hours()), ShouldEqual, 24)

			now = time.Now()
			model.Conditions.EndDate = now.Add(24 * 60 * time.Hour)
			model.State = api.RtoState_RTO_STATE_ACTIVE
			err = repository.MustUpdate(ctx, model)
			So(err, ShouldBeNil)

			res, err = repository.FindAllByFilter(ctx, &rto.Filter{
				States:           []api.RtoState{model.State},
				NotificationType: rto.NotificationEndOfTenancyDateBeforeTwoMonths,
				DateFrom:         now.Add(24 * 58.5 * time.Hour),
				DateTo:           now.Add(24 * 59.5 * time.Hour),
				DateColumnName:   rto.DateColumnNameEndDate,
			})
			So(err, ShouldBeNil)
			So(res, ShouldBeEmpty)

			res, err = repository.FindAllByFilter(ctx, &rto.Filter{
				States:           []api.RtoState{model.State},
				NotificationType: rto.NotificationEndOfTenancyDateBeforeTwoMonths,
				DateFrom:         now.Add(24 * 59.5 * time.Hour),
				DateTo:           now.Add(24 * 60.5 * time.Hour),
				DateColumnName:   rto.DateColumnNameEndDate,
			})
			So(err, ShouldBeNil)
			So(len(res), ShouldEqual, 1)
			So(res[0].ID, ShouldEqual, model.ID)

			res, err = repository.FindAllByFilter(ctx, &rto.Filter{
				States:           []api.RtoState{model.State},
				NotificationType: rto.NotificationEndOfTenancyDateBeforeTwoMonths,
				DateFrom:         now.Add(24 * 60.5 * time.Hour),
				DateTo:           now.Add(24 * 61.5 * time.Hour),
				DateColumnName:   rto.DateColumnNameEndDate,
			})
			So(err, ShouldBeNil)
			So(res, ShouldBeEmpty)

			res, err = repository.FindAllByFilter(ctx, &rto.Filter{
				States:           []api.RtoState{model.State},
				NotificationType: rto.NotificationEndOfTenancyDateBeforeTwoMonths,
				DateFrom:         now.Add(24 * 61.5 * time.Hour),
				DateTo:           now.Add(24 * 62.5 * time.Hour),
				DateColumnName:   rto.DateColumnNameEndDate,
			})
			So(err, ShouldBeNil)
			So(res, ShouldBeEmpty)

			propertyID := "korshiler-cherti"
			rtoByPropertyID, err := repository.FindByPropertyID(ctx, propertyID)
			So(err, ShouldBeError)
			So(rtoByPropertyID, ShouldBeNil)

			model.Property = propertyID
			err = repository.MustUpdate(ctx, model)
			So(err, ShouldBeNil)

			rtoByPropertyID, err = repository.FindByPropertyID(ctx, propertyID)
			So(err, ShouldBeNil)
			So(rtoByPropertyID, ShouldNotBeEmpty)
			So(rtoByPropertyID.ID, ShouldEqual, model.ID)
		})
	}
}
