package grpc

import (
	loggerservice "gitlab.mashplace.com/mashplace/backend/common/lib/logger/service"
	"gitlab.mashplace.com/mashplace/backend/offer/internal/app/middleware/transport/grpc/rtoserver"
	"gitlab.mashplace.com/mashplace/backend/offer/internal/app/transport/grpc"
	usecase "gitlab.mashplace.com/mashplace/backend/offer/internal/app/usecase/rto"
)

func NewRtoServer(logger loggerservice.Logger,
	createUseCase usecase.CreateUseCase,
	createByGuestUseCase usecase.CreateByGuestUseCase,
	updateLandlordUseCase usecase.UpdateLandlordUseCase,
	createTenancyUseCase usecase.CreateTenancyUseCase,
	updateRentPlanUseCase usecase.UpdateRentPlanUseCase,
	updateEndDateUseCase usecase.UpdateEndDateUseCase,
	renewalUseCase usecase.RenewalUseCase,
	updateStateToActiveUseCase usecase.UpdateStateToActiveUseCase,
	updateStateToWaitingForPaymentUseCase usecase.UpdateStateToWaitingForPaymentUseCase,
	updateStateToCancelledUseCase usecase.UpdateStateToCancelledUseCase,
	getByLandlordIDUseCase usecase.GetByLandlordIDUseCase,
	getByPropertyIDUseCase usecase.GetByPropertyIDUseCase,
	getByPropertyIDsUseCase usecase.GetByPropertyIDsUseCase,
	answerFromTenantUseCase usecase.AnswerFromTenantUseCase,
	landlordApprovedUseCase usecase.LandlordApprovedUseCase,
	rentPaymentUseCase usecase.RentPaymentUseCase,
	updateLeadTenantUseCase usecase.UpdateLeadTenantUseCase,
	updateLeadTenantUserUseCase usecase.UpdateLeadTenantUserUseCase) grpc.RtoServer {
	interfaceName := "RtoServer"
	grpcServer := grpc.NewRtoServer(createUseCase,
		createByGuestUseCase,
		updateLandlordUseCase,
		createTenancyUseCase,
		updateRentPlanUseCase,
		updateEndDateUseCase,
		renewalUseCase,
		updateStateToActiveUseCase,
		updateStateToWaitingForPaymentUseCase,
		updateStateToCancelledUseCase,
		getByLandlordIDUseCase,
		getByPropertyIDUseCase,
		getByPropertyIDsUseCase,
		answerFromTenantUseCase,
		landlordApprovedUseCase,
		rentPaymentUseCase,
		updateLeadTenantUseCase,
		updateLeadTenantUserUseCase)
	grpcServer = rtoserver.NewLoggingMiddleware(grpcServer, interfaceName, logger)
	grpcServer = rtoserver.NewTracingMiddleware(grpcServer, interfaceName)

	return grpcServer
}
