package history

import (
	"context"
	"time"

	"gitlab.mashplace.com/mashplace/backend/common/generic/log"
	"gitlab.mashplace.com/mashplace/backend/common/lib/constants"
	loggerservice "gitlab.mashplace.com/mashplace/backend/common/lib/logger/service"
	domain "gitlab.mashplace.com/mashplace/backend/offer/internal/app/domain/rto"

	"github.com/davecgh/go-spew/spew"
	"go.mongodb.org/mongo-driver/bson/primitive"
)

type loggingMiddleware struct {
	next          domain.HistoryRepository
	interfaceName string
	logger        loggerservice.Logger
}

func NewLoggingMiddleware(s domain.HistoryRepository,
	interfaceName string, logger loggerservice.Logger) domain.HistoryRepository {
	return &loggingMiddleware{
		next:          s,
		interfaceName: interfaceName,
		logger:        logger,
	}
}

func (s *loggingMiddleware) Create(ctx context.Context, history *domain.History) (err error) {
	defer func(begin time.Time) {
		log.WithContext(ctx).WithFields(log.Fields{
			"method":              s.interfaceName + "_" + "Create",
			"history":             spew.Sdump(history),
			"took":                time.Since(begin),
			constants.LogErrorKey: err,
		}).Debug()
	}(time.Now())

	return s.next.Create(ctx, history)
}

func (s *loggingMiddleware) DeleteByRtoID(ctx context.Context, rtoID primitive.ObjectID) (
	deletedCount int64, err error) {
	defer func(begin time.Time) {
		log.WithContext(ctx).WithFields(log.Fields{
			"method":              s.interfaceName + "_" + "DeleteByRtoID",
			"rtoID":               rtoID.Hex(),
			"deletedCount":        deletedCount,
			"took":                time.Since(begin),
			constants.LogErrorKey: err,
		}).Debug()
	}(time.Now())

	return s.next.DeleteByRtoID(ctx, rtoID)
}
