package history

import (
	"context"

	domain "gitlab.mashplace.com/mashplace/backend/offer/internal/app/domain/rto"

	"github.com/opentracing/opentracing-go"
	"go.mongodb.org/mongo-driver/bson/primitive"
)

type tracingMiddleware struct {
	next               domain.HistoryRepository
	middlewareAbleName string
}

func NewTracingMiddleware(s domain.HistoryRepository, middlewareAbleName string) domain.HistoryRepository {
	return &tracingMiddleware{
		next:               s,
		middlewareAbleName: middlewareAbleName,
	}
}

func (s *tracingMiddleware) Create(ctx context.Context, history *domain.History) error {
	span, ctx := opentracing.StartSpanFromContext(
		ctx, s.middlewareAbleName+" Create")
	defer span.Finish()

	return s.next.Create(ctx, history)
}

func (s *tracingMiddleware) DeleteByRtoID(ctx context.Context, rtoID primitive.ObjectID) (int64, error) {
	span, ctx := opentracing.StartSpanFromContext(
		ctx, s.middlewareAbleName+" DeleteByRtoID")
	defer span.Finish()

	return s.next.DeleteByRtoID(ctx, rtoID)
}
