package rto

import (
	"context"

	api "gitlab.mashplace.com/mashplace/backend/offer/api/rto"
	domain "gitlab.mashplace.com/mashplace/backend/offer/internal/app/domain/rto"

	"github.com/opentracing/opentracing-go"
	"go.mongodb.org/mongo-driver/bson/primitive"
)

type tracingMiddleware struct {
	next               domain.Repository
	middlewareAbleName string
}

func NewTracingMiddleware(s domain.Repository, middlewareAbleName string) domain.Repository {
	return &tracingMiddleware{
		next:               s,
		middlewareAbleName: middlewareAbleName,
	}
}

func (s *tracingMiddleware) Create(ctx context.Context, rto *domain.Rto) error {
	span, ctx := opentracing.StartSpanFromContext(
		ctx, s.middlewareAbleName+" Create")
	defer span.Finish()

	return s.next.Create(ctx, rto)
}

func (s *tracingMiddleware) Update(ctx context.Context, rto *domain.Rto) (int64, error) {
	span, ctx := opentracing.StartSpanFromContext(
		ctx, s.middlewareAbleName+" Update")
	defer span.Finish()

	return s.next.Update(ctx, rto)
}

func (s *tracingMiddleware) MustUpdate(ctx context.Context, rto *domain.Rto) error {
	span, ctx := opentracing.StartSpanFromContext(
		ctx, s.middlewareAbleName+" MustUpdate")
	defer span.Finish()

	return s.next.MustUpdate(ctx, rto)
}

func (s *tracingMiddleware) DeleteByID(ctx context.Context, id primitive.ObjectID) (int64, error) {
	span, ctx := opentracing.StartSpanFromContext(
		ctx, s.middlewareAbleName+" DeleteByID")
	defer span.Finish()

	return s.next.DeleteByID(ctx, id)
}

func (s *tracingMiddleware) Drop(ctx context.Context) error {
	span, ctx := opentracing.StartSpanFromContext(
		ctx, s.middlewareAbleName+" Drop")
	defer span.Finish()

	return s.next.Drop(ctx)
}

func (s *tracingMiddleware) FindByID(ctx context.Context, id primitive.ObjectID) (*domain.Rto, error) {
	span, ctx := opentracing.StartSpanFromContext(
		ctx, s.middlewareAbleName+" FindByID")
	defer span.Finish()

	return s.next.FindByID(ctx, id)
}

func (s *tracingMiddleware) FindByPropertyID(ctx context.Context, propertyID string) (*domain.Rto, error) {
	span, ctx := opentracing.StartSpanFromContext(
		ctx, s.middlewareAbleName+" FindByPropertyID")
	defer span.Finish()

	return s.next.FindByPropertyID(ctx, propertyID)
}

func (s *tracingMiddleware) FindAllByFilter(ctx context.Context, filter *domain.Filter) ([]*domain.Rto, error) {
	span, ctx := opentracing.StartSpanFromContext(
		ctx, s.middlewareAbleName+" FindAllByFilter")
	defer span.Finish()

	return s.next.FindAllByFilter(ctx, filter)
}

func (s *tracingMiddleware) FindAllEmptyLeadTenantUsersByStates(ctx context.Context,
	states []api.RtoState) ([]*domain.Rto, error) {
	span, ctx := opentracing.StartSpanFromContext(
		ctx, s.middlewareAbleName+" FindAllEmptyLeadTenantUsersByStates")
	defer span.Finish()

	return s.next.FindAllEmptyLeadTenantUsersByStates(ctx, states)
}

func (s *tracingMiddleware) FindAllEmptyLeadTenantUsersByEmail(ctx context.Context, leadTenantEmail string) (
	[]*domain.Rto, error) {
	span, ctx := opentracing.StartSpanFromContext(
		ctx, s.middlewareAbleName+" FindAllEmptyLeadTenantUsersByEmail")
	defer span.Finish()

	return s.next.FindAllEmptyLeadTenantUsersByEmail(ctx, leadTenantEmail)
}

func (s *tracingMiddleware) FindAllByPropertyIDs(ctx context.Context,
	filter *domain.AllByPropertyIDsFilter) ([]*domain.Rto, error) {
	span, ctx := opentracing.StartSpanFromContext(
		ctx, s.middlewareAbleName+" FindAllByPropertyIDs")
	defer span.Finish()

	return s.next.FindAllByPropertyIDs(ctx, filter)
}

func (s *tracingMiddleware) FindAllByLandlordID(ctx context.Context,
	filter *domain.AllByLandlordIDFilter) ([]*domain.Rto, error) {
	span, ctx := opentracing.StartSpanFromContext(
		ctx, s.middlewareAbleName+" FindAllByLandlordID")
	defer span.Finish()

	return s.next.FindAllByLandlordID(ctx, filter)
}
