package account

import (
	"context"

	account "gitlab.mashplace.com/mashplace/backend/account/api"
	request "gitlab.mashplace.com/mashplace/backend/offer/internal/app/domain/rto/resource/request/service"
	appservice "gitlab.mashplace.com/mashplace/backend/offer/internal/app/service"

	"github.com/opentracing/opentracing-go"
)

type tracingMiddleware struct {
	next               appservice.AccountService
	middlewareAbleName string
}

func NewTracingMiddleware(s appservice.AccountService, middlewareAbleName string) appservice.AccountService {
	return &tracingMiddleware{
		next:               s,
		middlewareAbleName: middlewareAbleName,
	}
}

func (s *tracingMiddleware) IsVeriffValid(ctx context.Context, req *request.IsVeriffValidRequest) error {
	span, ctx := opentracing.StartSpanFromContext(
		ctx, s.middlewareAbleName+" IsVeriffValid")
	defer span.Finish()

	return s.next.IsVeriffValid(ctx, req)
}

func (s *tracingMiddleware) IsOwnershipVerified(ctx context.Context, req *request.IsOwnershipVerifiedRequest) error {
	span, ctx := opentracing.StartSpanFromContext(
		ctx, s.middlewareAbleName+" IsOwnershipVerified")
	defer span.Finish()

	return s.next.IsOwnershipVerified(ctx, req)
}

func (s *tracingMiddleware) GetAccountByID(ctx context.Context, id string) (*account.Account, error) {
	span, ctx := opentracing.StartSpanFromContext(
		ctx, s.middlewareAbleName+" GetAccountByID")
	defer span.Finish()

	return s.next.GetAccountByID(ctx, id)
}

func (s *tracingMiddleware) GetAccountByEmail(ctx context.Context, email string) (*account.Account, error) {
	span, ctx := opentracing.StartSpanFromContext(
		ctx, s.middlewareAbleName+" GetAccountByEmail")
	defer span.Finish()

	return s.next.GetAccountByEmail(ctx, email)
}
