package extras

import (
	"context"
	"time"

	"gitlab.mashplace.com/mashplace/backend/common/generic/log"
	"gitlab.mashplace.com/mashplace/backend/common/lib/constants"
	loggerservice "gitlab.mashplace.com/mashplace/backend/common/lib/logger/service"
	extras "gitlab.mashplace.com/mashplace/backend/extras/api"
	request "gitlab.mashplace.com/mashplace/backend/offer/internal/app/domain/rto/resource/request/service"
	appservice "gitlab.mashplace.com/mashplace/backend/offer/internal/app/service"

	"github.com/davecgh/go-spew/spew"
)

type loggingMiddleware struct {
	next          appservice.ExtrasService
	interfaceName string
	logger        loggerservice.Logger
}

func NewLoggingMiddleware(s appservice.ExtrasService,
	interfaceName string, logger loggerservice.Logger) appservice.ExtrasService {
	return &loggingMiddleware{
		next:          s,
		interfaceName: interfaceName,
		logger:        logger,
	}
}

func (s *loggingMiddleware) GetServicePriceSnapshot(ctx context.Context, req *request.GetServicePriceSnapshotRequest) (
	resp *extras.ServiceSnapshot, err error) {
	defer func(begin time.Time) {
		log.WithContext(ctx).WithFields(log.Fields{
			"method":              s.interfaceName + "_" + "GetServicePriceSnapshot",
			"req":                 spew.Sdump(req),
			"resp":                spew.Sdump(resp),
			"took":                time.Since(begin),
			constants.LogErrorKey: err,
		}).Debug()
	}(time.Now())

	return s.next.GetServicePriceSnapshot(ctx, req)
}
