package invoice

import (
	"context"
	"time"

	"gitlab.mashplace.com/mashplace/backend/common/generic/log"
	"gitlab.mashplace.com/mashplace/backend/common/lib/constants"
	loggerservice "gitlab.mashplace.com/mashplace/backend/common/lib/logger/service"
	request "gitlab.mashplace.com/mashplace/backend/offer/internal/app/domain/rto/resource/request/service"
	appservice "gitlab.mashplace.com/mashplace/backend/offer/internal/app/service"

	"github.com/davecgh/go-spew/spew"
)

type loggingMiddleware struct {
	next          appservice.InvoiceService
	interfaceName string
	logger        loggerservice.Logger
}

func NewLoggingMiddleware(s appservice.InvoiceService,
	interfaceName string, logger loggerservice.Logger) appservice.InvoiceService {
	return &loggingMiddleware{
		next:          s,
		interfaceName: interfaceName,
		logger:        logger,
	}
}

func (s *loggingMiddleware) CreateStandingOrderInvoice(ctx context.Context,
	req *request.CreateStandingOrderInvoiceRequest) (id string, err error) {
	defer func(begin time.Time) {
		log.WithContext(ctx).WithFields(log.Fields{
			"method":              s.interfaceName + "_" + "CreateStandingOrderInvoice",
			"req":                 spew.Sdump(req),
			"id":                  id,
			"took":                time.Since(begin),
			constants.LogErrorKey: err,
		}).Debug()
	}(time.Now())

	return s.next.CreateStandingOrderInvoice(ctx, req)
}
