package listing

import (
	"context"

	listing "gitlab.mashplace.com/mashplace/backend/listing/api"
	request "gitlab.mashplace.com/mashplace/backend/offer/internal/app/domain/rto/resource/request/service"
	appservice "gitlab.mashplace.com/mashplace/backend/offer/internal/app/service"

	"github.com/opentracing/opentracing-go"
)

type tracingMiddleware struct {
	next               appservice.ListingService
	middlewareAbleName string
}

func NewTracingMiddleware(s appservice.ListingService, middlewareAbleName string) appservice.ListingService {
	return &tracingMiddleware{
		next:               s,
		middlewareAbleName: middlewareAbleName,
	}
}

func (s *tracingMiddleware) GetVersion(ctx context.Context, req *request.ListingGetVersionRequest) (
	*listing.Version, error) {
	span, ctx := opentracing.StartSpanFromContext(
		ctx, s.middlewareAbleName+" GetVersion")
	defer span.Finish()

	return s.next.GetVersion(ctx, req)
}

func (s *tracingMiddleware) VersionHasNotEmptyBankDetails(ctx context.Context, version *listing.Version) bool {
	span, ctx := opentracing.StartSpanFromContext(
		ctx, s.middlewareAbleName+" VersionHasNotEmptyBankDetails")
	defer span.Finish()

	return s.next.VersionHasNotEmptyBankDetails(ctx, version)
}

func (s *tracingMiddleware) UpdateTenantID(ctx context.Context, req *request.ListingUpdateTenantIDRequest) error {
	span, ctx := opentracing.StartSpanFromContext(
		ctx, s.middlewareAbleName+" UpdateTenantID")
	defer span.Finish()

	return s.next.UpdateTenantID(ctx, req)
}

func (s *tracingMiddleware) UpdateVersionStatus(ctx context.Context,
	req *request.ListingUpdateVersionStatusRequest) error {
	span, ctx := opentracing.StartSpanFromContext(
		ctx, s.middlewareAbleName+" UpdateVersionStatus")
	defer span.Finish()

	return s.next.UpdateVersionStatus(ctx, req)
}
