package closecom

import (
	"context"
	"time"

	"gitlab.mashplace.com/mashplace/backend/common/generic/log"
	"gitlab.mashplace.com/mashplace/backend/common/lib/constants"
	loggerservice "gitlab.mashplace.com/mashplace/backend/common/lib/logger/service"
	request "gitlab.mashplace.com/mashplace/backend/offer/internal/app/domain/rto/resource/request/service/rto"
	response "gitlab.mashplace.com/mashplace/backend/offer/internal/app/domain/rto/resource/response/service/rto"
	service "gitlab.mashplace.com/mashplace/backend/offer/internal/app/service/rto"

	"github.com/davecgh/go-spew/spew"
)

type loggingMiddleware struct {
	next          service.CloseComService
	interfaceName string
	logger        loggerservice.Logger
}

func NewLoggingMiddleware(s service.CloseComService,
	interfaceName string, logger loggerservice.Logger) service.CloseComService {
	return &loggingMiddleware{
		next:          s,
		interfaceName: interfaceName,
		logger:        logger,
	}
}

func (s *loggingMiddleware) SaveOpportunity(ctx context.Context, req *request.CloseComSaveOpportunityRequest) (
	resp *response.CloseComSaveOpportunityResponse, err error) {
	defer func(begin time.Time) {
		log.WithContext(ctx).WithFields(log.Fields{
			"method":              s.interfaceName + "_" + "SaveOpportunity",
			"req":                 spew.Sdump(req),
			"resp":                spew.Sdump(resp),
			"took":                time.Since(begin),
			constants.LogErrorKey: err,
		}).Debug()
	}(time.Now())

	return s.next.SaveOpportunity(ctx, req)
}

func (s *loggingMiddleware) DeleteOpportunity(ctx context.Context, id string) (err error) {
	defer func(begin time.Time) {
		log.WithContext(ctx).WithFields(log.Fields{
			"method":              s.interfaceName + "_" + "DeleteOpportunity",
			"id":                  id,
			"took":                time.Since(begin),
			constants.LogErrorKey: err,
		}).Debug()
	}(time.Now())

	return s.next.DeleteOpportunity(ctx, id)
}

func (s *loggingMiddleware) FindLead(ctx context.Context, req *request.CloseComFindLeadRequest) (id string, err error) {
	defer func(begin time.Time) {
		log.WithContext(ctx).WithFields(log.Fields{
			"method":              s.interfaceName + "_" + "FindLead",
			"req":                 spew.Sdump(req),
			"id":                  id,
			"took":                time.Since(begin),
			constants.LogErrorKey: err,
		}).Debug()
	}(time.Now())

	return s.next.FindLead(ctx, req)
}

func (s *loggingMiddleware) GetLead(ctx context.Context, id string) (
	resp *response.CloseComGetLeadResponse, err error) {
	defer func(begin time.Time) {
		log.WithContext(ctx).WithFields(log.Fields{
			"method":              s.interfaceName + "_" + "GetLead",
			"id":                  id,
			"resp":                spew.Sdump(resp),
			"took":                time.Since(begin),
			constants.LogErrorKey: err,
		}).Debug()
	}(time.Now())

	return s.next.GetLead(ctx, id)
}

func (s *loggingMiddleware) CreateLead(ctx context.Context, req *request.CloseComCreateLeadRequest) (
	id string, err error) {
	defer func(begin time.Time) {
		log.WithContext(ctx).WithFields(log.Fields{
			"method":              s.interfaceName + "_" + "CreateLead",
			"req":                 spew.Sdump(req),
			"id":                  id,
			"took":                time.Since(begin),
			constants.LogErrorKey: err,
		}).Debug()
	}(time.Now())

	return s.next.CreateLead(ctx, req)
}

func (s *loggingMiddleware) DeleteLead(ctx context.Context, id string) (err error) {
	defer func(begin time.Time) {
		log.WithContext(ctx).WithFields(log.Fields{
			"method":              s.interfaceName + "_" + "UpdateStateToWaitingForPayment",
			"id":                  id,
			"took":                time.Since(begin),
			constants.LogErrorKey: err,
		}).Debug()
	}(time.Now())

	return s.next.DeleteLead(ctx, id)
}
