package closecom

import (
	"context"

	request "gitlab.mashplace.com/mashplace/backend/offer/internal/app/domain/rto/resource/request/service/rto"
	response "gitlab.mashplace.com/mashplace/backend/offer/internal/app/domain/rto/resource/response/service/rto"
	service "gitlab.mashplace.com/mashplace/backend/offer/internal/app/service/rto"

	"github.com/opentracing/opentracing-go"
)

type tracingMiddleware struct {
	next               service.CloseComService
	middlewareAbleName string
}

func NewTracingMiddleware(s service.CloseComService, middlewareAbleName string) service.CloseComService {
	return &tracingMiddleware{
		next:               s,
		middlewareAbleName: middlewareAbleName,
	}
}

func (s *tracingMiddleware) SaveOpportunity(ctx context.Context, req *request.CloseComSaveOpportunityRequest) (
	*response.CloseComSaveOpportunityResponse, error) {
	span, ctx := opentracing.StartSpanFromContext(
		ctx, s.middlewareAbleName+" SaveOpportunity")
	defer span.Finish()

	return s.next.SaveOpportunity(ctx, req)
}

func (s *tracingMiddleware) DeleteOpportunity(ctx context.Context, id string) error {
	span, ctx := opentracing.StartSpanFromContext(
		ctx, s.middlewareAbleName+" DeleteOpportunity")
	defer span.Finish()

	return s.next.DeleteOpportunity(ctx, id)
}

func (s *tracingMiddleware) FindLead(ctx context.Context, req *request.CloseComFindLeadRequest) (string, error) {
	span, ctx := opentracing.StartSpanFromContext(
		ctx, s.middlewareAbleName+" FindLead")
	defer span.Finish()

	return s.next.FindLead(ctx, req)
}

func (s *tracingMiddleware) GetLead(ctx context.Context, id string) (*response.CloseComGetLeadResponse, error) {
	span, ctx := opentracing.StartSpanFromContext(
		ctx, s.middlewareAbleName+" GetLead")
	defer span.Finish()

	return s.next.GetLead(ctx, id)
}

func (s *tracingMiddleware) CreateLead(ctx context.Context, req *request.CloseComCreateLeadRequest) (string, error) {
	span, ctx := opentracing.StartSpanFromContext(
		ctx, s.middlewareAbleName+" CreateLead")
	defer span.Finish()

	return s.next.CreateLead(ctx, req)
}

func (s *tracingMiddleware) DeleteLead(ctx context.Context, id string) error {
	span, ctx := opentracing.StartSpanFromContext(
		ctx, s.middlewareAbleName+" DeleteLead")
	defer span.Finish()

	return s.next.DeleteLead(ctx, id)
}
