package finish

import (
	"context"
	"time"

	"gitlab.mashplace.com/mashplace/backend/common/generic/log"

	"gitlab.mashplace.com/mashplace/backend/common/lib/constants"
	loggerservice "gitlab.mashplace.com/mashplace/backend/common/lib/logger/service"
	"gitlab.mashplace.com/mashplace/backend/offer/internal/app/domain/rto/resource/response"
	service "gitlab.mashplace.com/mashplace/backend/offer/internal/app/service/rto/daemon"

	"github.com/davecgh/go-spew/spew"
)

type loggingMiddleware struct {
	next          service.FinishService
	interfaceName string
	logger        loggerservice.Logger
}

func NewLoggingMiddleware(s service.FinishService,
	interfaceName string, logger loggerservice.Logger) service.FinishService {
	return &loggingMiddleware{
		next:          s,
		interfaceName: interfaceName,
		logger:        logger,
	}
}

func (s *loggingMiddleware) FinishAll(ctx context.Context) (err error) {
	defer func(begin time.Time) {
		log.WithContext(ctx).WithFields(log.Fields{
			"method":              s.interfaceName + "_" + "FinishAll",
			"took":                time.Since(begin),
			constants.LogErrorKey: err,
		}).Debug()
	}(time.Now())

	return s.next.FinishAll(ctx)
}

func (s *loggingMiddleware) Finish(ctx context.Context) (resp *response.IDsResponse, err error) {
	defer func(begin time.Time) {
		log.WithContext(ctx).WithFields(log.Fields{
			"method":              s.interfaceName + "_" + "Finish",
			"resp":                spew.Sdump(resp),
			"took":                time.Since(begin),
			constants.LogErrorKey: err,
		}).Debug()
	}(time.Now())

	return s.next.Finish(ctx)
}
