package finish

import (
	"context"

	"gitlab.mashplace.com/mashplace/backend/offer/internal/app/domain/rto/resource/response"
	service "gitlab.mashplace.com/mashplace/backend/offer/internal/app/service/rto/daemon"

	"github.com/opentracing/opentracing-go"
)

type tracingMiddleware struct {
	next               service.FinishService
	middlewareAbleName string
}

func NewTracingMiddleware(s service.FinishService, middlewareAbleName string) service.FinishService {
	return &tracingMiddleware{
		next:               s,
		middlewareAbleName: middlewareAbleName,
	}
}

func (s *tracingMiddleware) FinishAll(ctx context.Context) error {
	span, ctx := opentracing.StartSpanFromContext(
		ctx, s.middlewareAbleName+" FinishAll")
	defer span.Finish()

	return s.next.FinishAll(ctx)
}

func (s *tracingMiddleware) Finish(ctx context.Context) (*response.IDsResponse, error) {
	span, ctx := opentracing.StartSpanFromContext(
		ctx, s.middlewareAbleName+" Finish")
	defer span.Finish()

	return s.next.Finish(ctx)
}
