package notification

import (
	"context"
	"time"

	"gitlab.mashplace.com/mashplace/backend/common/generic/log"
	"gitlab.mashplace.com/mashplace/backend/common/lib/constants"
	loggerservice "gitlab.mashplace.com/mashplace/backend/common/lib/logger/service"
	request "gitlab.mashplace.com/mashplace/backend/offer/internal/app/domain/rto/resource/request/service/rto/daemon"
	"gitlab.mashplace.com/mashplace/backend/offer/internal/app/domain/rto/resource/response"
	service "gitlab.mashplace.com/mashplace/backend/offer/internal/app/service/rto/daemon"

	"github.com/davecgh/go-spew/spew"
)

type loggingMiddleware struct {
	next          service.NotificationService
	interfaceName string
	logger        loggerservice.Logger
}

func NewLoggingMiddleware(s service.NotificationService,
	interfaceName string, logger loggerservice.Logger) service.NotificationService {
	return &loggingMiddleware{
		next:          s,
		interfaceName: interfaceName,
		logger:        logger,
	}
}

func (s *loggingMiddleware) RemindAll(ctx context.Context) (err error) {
	defer func(begin time.Time) {
		log.WithContext(ctx).WithFields(log.Fields{
			"method":              s.interfaceName + "_" + "RemindAll",
			"took":                time.Since(begin),
			constants.LogErrorKey: err,
		}).Debug()
	}(time.Now())

	return s.next.RemindAll(ctx)
}

func (s *loggingMiddleware) RemindAboutPendingSetup(ctx context.Context) (err error) {
	defer func(begin time.Time) {
		log.WithContext(ctx).WithFields(log.Fields{
			"method":              s.interfaceName + "_" + "RemindAboutPendingSetup",
			"took":                time.Since(begin),
			constants.LogErrorKey: err,
		}).Debug()
	}(time.Now())

	return s.next.RemindAboutPendingSetup(ctx)
}

func (s *loggingMiddleware) SendRemindAboutPendingSetup(ctx context.Context,
	req *request.SendRemindAboutPendingSetupRequest) (resp *response.IDsResponse, err error) {
	defer func(begin time.Time) {
		log.WithContext(ctx).WithFields(log.Fields{
			"method":              s.interfaceName + "_" + "SendRemindAboutPendingSetup",
			"req":                 spew.Sdump(req),
			"resp":                spew.Sdump(resp),
			"took":                time.Since(begin),
			constants.LogErrorKey: err,
		}).Debug()
	}(time.Now())

	return s.next.SendRemindAboutPendingSetup(ctx, req)
}

func (s *loggingMiddleware) RemindAboutWaitingForStandingOrderSetup(ctx context.Context) (err error) {
	defer func(begin time.Time) {
		log.WithContext(ctx).WithFields(log.Fields{
			"method":              s.interfaceName + "_" + "RemindAboutWaitingForStandingOrderSetup",
			"took":                time.Since(begin),
			constants.LogErrorKey: err,
		}).Debug()
	}(time.Now())

	return s.next.RemindAboutWaitingForStandingOrderSetup(ctx)
}

func (s *loggingMiddleware) SendRemindAboutWaitingForStandingOrderSetup(ctx context.Context,
	req *request.SendRemindAboutWaitingForStandingOrderSetupRequest) (resp *response.IDsResponse, err error) {
	defer func(begin time.Time) {
		log.WithContext(ctx).WithFields(log.Fields{
			"method":              s.interfaceName + "_" + "SendRemindAboutWaitingForStandingOrderSetup",
			"req":                 spew.Sdump(req),
			"resp":                spew.Sdump(resp),
			"took":                time.Since(begin),
			constants.LogErrorKey: err,
		}).Debug()
	}(time.Now())

	return s.next.SendRemindAboutWaitingForStandingOrderSetup(ctx, req)
}

func (s *loggingMiddleware) RemindAboutEndOfTenancyDate(ctx context.Context) (err error) {
	defer func(begin time.Time) {
		log.WithContext(ctx).WithFields(log.Fields{
			"method":              s.interfaceName + "_" + "RemindAboutEndOfTenancyDate",
			"took":                time.Since(begin),
			constants.LogErrorKey: err,
		}).Debug()
	}(time.Now())

	return s.next.RemindAboutEndOfTenancyDate(ctx)
}

func (s *loggingMiddleware) SendRemindAboutEndOfTenancyDate(ctx context.Context,
	req *request.SendRemindAboutEndOfTenancyDateRequest) (resp *response.IDsResponse, err error) {
	defer func(begin time.Time) {
		log.WithContext(ctx).WithFields(log.Fields{
			"method":              s.interfaceName + "_" + "SendRemindAboutEndOfTenancyDate",
			"req":                 spew.Sdump(req),
			"resp":                spew.Sdump(resp),
			"took":                time.Since(begin),
			constants.LogErrorKey: err,
		}).Debug()
	}(time.Now())

	return s.next.SendRemindAboutEndOfTenancyDate(ctx, req)
}
