package notification

import (
	"context"

	request "gitlab.mashplace.com/mashplace/backend/offer/internal/app/domain/rto/resource/request/service/rto/daemon"
	"gitlab.mashplace.com/mashplace/backend/offer/internal/app/domain/rto/resource/response"
	service "gitlab.mashplace.com/mashplace/backend/offer/internal/app/service/rto/daemon"

	"github.com/opentracing/opentracing-go"
)

type tracingMiddleware struct {
	next               service.NotificationService
	middlewareAbleName string
}

func NewTracingMiddleware(s service.NotificationService, middlewareAbleName string) service.NotificationService {
	return &tracingMiddleware{
		next:               s,
		middlewareAbleName: middlewareAbleName,
	}
}

func (s *tracingMiddleware) RemindAll(ctx context.Context) error {
	span, ctx := opentracing.StartSpanFromContext(
		ctx, s.middlewareAbleName+" RemindAll")
	defer span.Finish()

	return s.next.RemindAll(ctx)
}

func (s *tracingMiddleware) RemindAboutPendingSetup(ctx context.Context) error {
	span, ctx := opentracing.StartSpanFromContext(
		ctx, s.middlewareAbleName+" RemindAboutPendingSetup")
	defer span.Finish()

	return s.next.RemindAboutPendingSetup(ctx)
}

func (s *tracingMiddleware) SendRemindAboutPendingSetup(ctx context.Context,
	req *request.SendRemindAboutPendingSetupRequest) (*response.IDsResponse, error) {
	span, ctx := opentracing.StartSpanFromContext(
		ctx, s.middlewareAbleName+" SendRemindAboutPendingSetup")
	defer span.Finish()

	return s.next.SendRemindAboutPendingSetup(ctx, req)
}

func (s *tracingMiddleware) RemindAboutWaitingForStandingOrderSetup(ctx context.Context) error {
	span, ctx := opentracing.StartSpanFromContext(
		ctx, s.middlewareAbleName+" RemindAboutWaitingForStandingOrderSetup")
	defer span.Finish()

	return s.next.RemindAboutWaitingForStandingOrderSetup(ctx)
}

func (s *tracingMiddleware) SendRemindAboutWaitingForStandingOrderSetup(ctx context.Context,
	req *request.SendRemindAboutWaitingForStandingOrderSetupRequest) (*response.IDsResponse, error) {
	span, ctx := opentracing.StartSpanFromContext(
		ctx, s.middlewareAbleName+" SendRemindAboutWaitingForStandingOrderSetup")
	defer span.Finish()

	return s.next.SendRemindAboutWaitingForStandingOrderSetup(ctx, req)
}

func (s *tracingMiddleware) RemindAboutEndOfTenancyDate(ctx context.Context) error {
	span, ctx := opentracing.StartSpanFromContext(
		ctx, s.middlewareAbleName+" RemindAboutEndOfTenancyDate")
	defer span.Finish()

	return s.next.RemindAboutEndOfTenancyDate(ctx)
}

func (s *tracingMiddleware) SendRemindAboutEndOfTenancyDate(ctx context.Context,
	req *request.SendRemindAboutEndOfTenancyDateRequest) (*response.IDsResponse, error) {
	span, ctx := opentracing.StartSpanFromContext(
		ctx, s.middlewareAbleName+" SendRemindAboutEndOfTenancyDate")
	defer span.Finish()

	return s.next.SendRemindAboutEndOfTenancyDate(ctx, req)
}
