package updateleadtenantuser

import (
	"context"
	"time"

	"gitlab.mashplace.com/mashplace/backend/common/generic/log"
	"gitlab.mashplace.com/mashplace/backend/common/lib/constants"
	loggerservice "gitlab.mashplace.com/mashplace/backend/common/lib/logger/service"
	"gitlab.mashplace.com/mashplace/backend/offer/internal/app/domain/rto/resource/response"
	service "gitlab.mashplace.com/mashplace/backend/offer/internal/app/service/rto/daemon"

	"github.com/davecgh/go-spew/spew"
)

type loggingMiddleware struct {
	next          service.UpdateLeadTenantUserService
	interfaceName string
	logger        loggerservice.Logger
}

func NewLoggingMiddleware(s service.UpdateLeadTenantUserService,
	interfaceName string, logger loggerservice.Logger) service.UpdateLeadTenantUserService {
	return &loggingMiddleware{
		next:          s,
		interfaceName: interfaceName,
		logger:        logger,
	}
}

func (s *loggingMiddleware) UpdateAllLeadTenantUsers(ctx context.Context) (err error) {
	defer func(begin time.Time) {
		log.WithContext(ctx).WithFields(log.Fields{
			"method":              s.interfaceName + "_" + "UpdateAllLeadTenantUsers",
			"took":                time.Since(begin),
			constants.LogErrorKey: err,
		}).Debug()
	}(time.Now())

	return s.next.UpdateAllLeadTenantUsers(ctx)
}

func (s *loggingMiddleware) UpdateLeadTenantUsers(ctx context.Context) (resp *response.IDsResponse, err error) {
	defer func(begin time.Time) {
		log.WithContext(ctx).WithFields(log.Fields{
			"method":              s.interfaceName + "_" + "UpdateLeadTenantUsers",
			"resp":                spew.Sdump(resp),
			"took":                time.Since(begin),
			constants.LogErrorKey: err,
		}).Debug()
	}(time.Now())

	return s.next.UpdateLeadTenantUsers(ctx)
}

func (s *loggingMiddleware) UpdateLeadTenantUsersInListing(ctx context.Context) (
	resp *response.IDsResponse, err error) {
	defer func(begin time.Time) {
		log.WithContext(ctx).WithFields(log.Fields{
			"method":              s.interfaceName + "_" + "UpdateLeadTenantUsersInListing",
			"resp":                spew.Sdump(resp),
			"took":                time.Since(begin),
			constants.LogErrorKey: err,
		}).Debug()
	}(time.Now())

	return s.next.UpdateLeadTenantUsersInListing(ctx)
}
