package updateleadtenantuser

import (
	"context"

	"gitlab.mashplace.com/mashplace/backend/offer/internal/app/domain/rto/resource/response"
	service "gitlab.mashplace.com/mashplace/backend/offer/internal/app/service/rto/daemon"

	"github.com/opentracing/opentracing-go"
)

type tracingMiddleware struct {
	next               service.UpdateLeadTenantUserService
	middlewareAbleName string
}

func NewTracingMiddleware(s service.UpdateLeadTenantUserService,
	middlewareAbleName string) service.UpdateLeadTenantUserService {
	return &tracingMiddleware{
		next:               s,
		middlewareAbleName: middlewareAbleName,
	}
}

func (s *tracingMiddleware) UpdateAllLeadTenantUsers(ctx context.Context) error {
	span, ctx := opentracing.StartSpanFromContext(
		ctx, s.middlewareAbleName+" UpdateAllLeadTenantUsers")
	defer span.Finish()

	return s.next.UpdateAllLeadTenantUsers(ctx)
}

func (s *tracingMiddleware) UpdateLeadTenantUsers(ctx context.Context) (*response.IDsResponse, error) {
	span, ctx := opentracing.StartSpanFromContext(
		ctx, s.middlewareAbleName+" UpdateLeadTenantUsers")
	defer span.Finish()

	return s.next.UpdateLeadTenantUsers(ctx)
}

func (s *tracingMiddleware) UpdateLeadTenantUsersInListing(ctx context.Context) (*response.IDsResponse, error) {
	span, ctx := opentracing.StartSpanFromContext(
		ctx, s.middlewareAbleName+" UpdateLeadTenantUsersInListing")
	defer span.Finish()

	return s.next.UpdateLeadTenantUsersInListing(ctx)
}
