package notification

import (
	"context"
	"time"

	"gitlab.mashplace.com/mashplace/backend/common/generic/log"
	"gitlab.mashplace.com/mashplace/backend/common/lib/constants"
	loggerservice "gitlab.mashplace.com/mashplace/backend/common/lib/logger/service"
	request "gitlab.mashplace.com/mashplace/backend/offer/internal/app/domain/rto/resource/request/service/rto"
	service "gitlab.mashplace.com/mashplace/backend/offer/internal/app/service/rto"

	"github.com/davecgh/go-spew/spew"
)

type loggingMiddleware struct {
	next          service.NotificationService
	interfaceName string
	logger        loggerservice.Logger
}

func NewLoggingMiddleware(s service.NotificationService,
	interfaceName string, logger loggerservice.Logger) service.NotificationService {
	return &loggingMiddleware{
		next:          s,
		interfaceName: interfaceName,
		logger:        logger,
	}
}

func (s *loggingMiddleware) SendWaitingForStandingOrderSetup(ctx context.Context,
	req *request.SendWaitingForStandingOrderSetupRequest) (err error) {
	defer func(begin time.Time) {
		log.WithContext(ctx).WithFields(log.Fields{
			"method":              s.interfaceName + "_" + "SendWaitingForStandingOrderSetup",
			"req":                 spew.Sdump(req),
			"took":                time.Since(begin),
			constants.LogErrorKey: err,
		}).Debug()
	}(time.Now())

	return s.next.SendWaitingForStandingOrderSetup(ctx, req)
}

func (s *loggingMiddleware) SendSetupComplete(ctx context.Context, req *request.SendSetupCompleteRequest) (err error) {
	defer func(begin time.Time) {
		log.WithContext(ctx).WithFields(log.Fields{
			"method":              s.interfaceName + "_" + "SendSetupComplete",
			"req":                 spew.Sdump(req),
			"took":                time.Since(begin),
			constants.LogErrorKey: err,
		}).Debug()
	}(time.Now())

	return s.next.SendSetupComplete(ctx, req)
}

func (s *loggingMiddleware) SendActive(ctx context.Context, req *request.SendActiveRequest) (err error) {
	defer func(begin time.Time) {
		log.WithContext(ctx).WithFields(log.Fields{
			"method":              s.interfaceName + "_" + "SendActive",
			"req":                 spew.Sdump(req),
			"took":                time.Since(begin),
			constants.LogErrorKey: err,
		}).Debug()
	}(time.Now())

	return s.next.SendActive(ctx, req)
}
