package notification

import (
	"context"

	request "gitlab.mashplace.com/mashplace/backend/offer/internal/app/domain/rto/resource/request/service/rto"
	service "gitlab.mashplace.com/mashplace/backend/offer/internal/app/service/rto"

	"github.com/opentracing/opentracing-go"
)

type tracingMiddleware struct {
	next               service.NotificationService
	middlewareAbleName string
}

func NewTracingMiddleware(s service.NotificationService, middlewareAbleName string) service.NotificationService {
	return &tracingMiddleware{
		next:               s,
		middlewareAbleName: middlewareAbleName,
	}
}

func (s *tracingMiddleware) SendWaitingForStandingOrderSetup(ctx context.Context,
	req *request.SendWaitingForStandingOrderSetupRequest) error {
	span, ctx := opentracing.StartSpanFromContext(
		ctx, s.middlewareAbleName+" SendWaitingForStandingOrderSetup")
	defer span.Finish()

	return s.next.SendWaitingForStandingOrderSetup(ctx, req)
}

func (s *tracingMiddleware) SendSetupComplete(ctx context.Context, req *request.SendSetupCompleteRequest) error {
	span, ctx := opentracing.StartSpanFromContext(
		ctx, s.middlewareAbleName+" SendSetupComplete")
	defer span.Finish()

	return s.next.SendSetupComplete(ctx, req)
}

func (s *tracingMiddleware) SendActive(ctx context.Context, req *request.SendActiveRequest) error {
	span, ctx := opentracing.StartSpanFromContext(
		ctx, s.middlewareAbleName+" SendActive")
	defer span.Finish()

	return s.next.SendActive(ctx, req)
}
