package security

import (
	"context"
	"time"

	"gitlab.mashplace.com/mashplace/backend/common/generic/log"
	"gitlab.mashplace.com/mashplace/backend/common/lib/constants"
	loggerservice "gitlab.mashplace.com/mashplace/backend/common/lib/logger/service"
	request "gitlab.mashplace.com/mashplace/backend/offer/internal/app/domain/rto/resource/request/service/rto"
	"gitlab.mashplace.com/mashplace/backend/offer/internal/app/domain/rto/resource/response"
	service "gitlab.mashplace.com/mashplace/backend/offer/internal/app/service/rto"

	"github.com/davecgh/go-spew/spew"
)

type loggingMiddleware struct {
	next          service.SecurityService
	interfaceName string
	logger        loggerservice.Logger
}

func NewLoggingMiddleware(s service.SecurityService,
	interfaceName string, logger loggerservice.Logger) service.SecurityService {
	return &loggingMiddleware{
		next:          s,
		interfaceName: interfaceName,
		logger:        logger,
	}
}

func (s *loggingMiddleware) CanLandlordUpdateRto(ctx context.Context,
	req *request.CanLandlordUpdateRtoRequest) (resp *response.RtoResponse, err error) {
	defer func(begin time.Time) {
		log.WithContext(ctx).WithFields(log.Fields{
			"method":              s.interfaceName + "_" + "CanLandlordUpdateRto",
			"req":                 spew.Sdump(req),
			"resp":                spew.Sdump(resp),
			"took":                time.Since(begin),
			constants.LogErrorKey: err,
		}).Debug()
	}(time.Now())

	return s.next.CanLandlordUpdateRto(ctx, req)
}

func (s *loggingMiddleware) CanUserReadRto(ctx context.Context, req *request.CanUserReadRtoRequest) (err error) {
	defer func(begin time.Time) {
		log.WithContext(ctx).WithFields(log.Fields{
			"method":              s.interfaceName + "_" + "CanUserReadRto",
			"req":                 spew.Sdump(req),
			"took":                time.Since(begin),
			constants.LogErrorKey: err,
		}).Debug()
	}(time.Now())

	return s.next.CanUserReadRto(ctx, req)
}

func (s *loggingMiddleware) CanTenantUpdateRto(ctx context.Context,
	req *request.CanTenantUpdateRtoRequest) (resp *response.RtoResponse, err error) {
	defer func(begin time.Time) {
		log.WithContext(ctx).WithFields(log.Fields{
			"method":              s.interfaceName + "_" + "CanTenantUpdateRto",
			"req":                 spew.Sdump(req),
			"resp":                spew.Sdump(resp),
			"took":                time.Since(begin),
			constants.LogErrorKey: err,
		}).Debug()
	}(time.Now())

	return s.next.CanTenantUpdateRto(ctx, req)
}
