package security

import (
	"context"

	request "gitlab.mashplace.com/mashplace/backend/offer/internal/app/domain/rto/resource/request/service/rto"
	"gitlab.mashplace.com/mashplace/backend/offer/internal/app/domain/rto/resource/response"
	service "gitlab.mashplace.com/mashplace/backend/offer/internal/app/service/rto"

	"github.com/opentracing/opentracing-go"
)

type tracingMiddleware struct {
	next               service.SecurityService
	middlewareAbleName string
}

func NewTracingMiddleware(s service.SecurityService, middlewareAbleName string) service.SecurityService {
	return &tracingMiddleware{
		next:               s,
		middlewareAbleName: middlewareAbleName,
	}
}

func (s *tracingMiddleware) CanLandlordUpdateRto(ctx context.Context,
	req *request.CanLandlordUpdateRtoRequest) (*response.RtoResponse, error) {
	span, ctx := opentracing.StartSpanFromContext(
		ctx, s.middlewareAbleName+" CanLandlordUpdateRto")
	defer span.Finish()

	return s.next.CanLandlordUpdateRto(ctx, req)
}

func (s *tracingMiddleware) CanUserReadRto(ctx context.Context, req *request.CanUserReadRtoRequest) error {
	span, ctx := opentracing.StartSpanFromContext(
		ctx, s.middlewareAbleName+" CanUserReadRto")
	defer span.Finish()

	return s.next.CanUserReadRto(ctx, req)
}

func (s *tracingMiddleware) CanTenantUpdateRto(ctx context.Context,
	req *request.CanTenantUpdateRtoRequest) (*response.RtoResponse, error) {
	span, ctx := opentracing.StartSpanFromContext(
		ctx, s.middlewareAbleName+" CanTenantUpdateRto")
	defer span.Finish()

	return s.next.CanTenantUpdateRto(ctx, req)
}
