package rtoserver

import (
	"context"

	api "gitlab.mashplace.com/mashplace/backend/offer/api/rto"
	"gitlab.mashplace.com/mashplace/backend/offer/internal/app/transport/grpc"

	"github.com/opentracing/opentracing-go"
)

type tracingMiddleware struct {
	api.UnimplementedRtoServiceServer
	next               grpc.RtoServer
	middlewareAbleName string
}

func NewTracingMiddleware(s grpc.RtoServer,
	middlewareAbleName string) grpc.RtoServer {
	return &tracingMiddleware{
		next:               s,
		middlewareAbleName: middlewareAbleName,
	}
}

func (s *tracingMiddleware) Create(ctx context.Context, req *api.CreateRequest) (*api.RtoResponse, error) {
	span, ctx := opentracing.StartSpanFromContext(
		ctx, s.middlewareAbleName+" Create")
	defer span.Finish()

	return s.next.Create(ctx, req)
}

func (s *tracingMiddleware) CreateByGuest(ctx context.Context, req *api.CreateByGuestRequest) (
	*api.RtoResponse, error) {
	span, ctx := opentracing.StartSpanFromContext(
		ctx, s.middlewareAbleName+" CreateByGuest")
	defer span.Finish()

	return s.next.CreateByGuest(ctx, req)
}

func (s *tracingMiddleware) UpdateLandlord(ctx context.Context, req *api.UpdateLandlordRequest) (
	*api.RtoResponse, error) {
	span, ctx := opentracing.StartSpanFromContext(
		ctx, s.middlewareAbleName+" UpdateLandlord")
	defer span.Finish()

	return s.next.UpdateLandlord(ctx, req)
}

func (s *tracingMiddleware) CreateTenancy(ctx context.Context, req *api.CreateTenancyRequest) (
	*api.RtoResponse, error) {
	span, ctx := opentracing.StartSpanFromContext(
		ctx, s.middlewareAbleName+" CreateTenancy")
	defer span.Finish()

	return s.next.CreateTenancy(ctx, req)
}

func (s *tracingMiddleware) UpdateRentPlan(ctx context.Context, req *api.UpdateRentPlanRequest) (
	*api.RtoResponse, error) {
	span, ctx := opentracing.StartSpanFromContext(
		ctx, s.middlewareAbleName+" UpdateRentPlan")
	defer span.Finish()

	return s.next.UpdateRentPlan(ctx, req)
}

func (s *tracingMiddleware) UpdateEndDate(ctx context.Context, req *api.UpdateEndDateRequest) (
	*api.RtoResponse, error) {
	span, ctx := opentracing.StartSpanFromContext(
		ctx, s.middlewareAbleName+" UpdateEndDate")
	defer span.Finish()

	return s.next.UpdateEndDate(ctx, req)
}

func (s *tracingMiddleware) Renewal(ctx context.Context, req *api.RenewalRequest) (*api.RtoResponse, error) {
	span, ctx := opentracing.StartSpanFromContext(
		ctx, s.middlewareAbleName+" Renewal")
	defer span.Finish()

	return s.next.Renewal(ctx, req)
}

func (s *tracingMiddleware) UpdateStateToActive(ctx context.Context,
	req *api.UpdateStateToActiveRequest) (*api.RtoResponse, error) {
	span, ctx := opentracing.StartSpanFromContext(
		ctx, s.middlewareAbleName+" UpdateStateToActive")
	defer span.Finish()

	return s.next.UpdateStateToActive(ctx, req)
}

func (s *tracingMiddleware) Cancel(ctx context.Context, req *api.CancelRequest) (*api.RtoResponse, error) {
	span, ctx := opentracing.StartSpanFromContext(
		ctx, s.middlewareAbleName+" Cancel")
	defer span.Finish()

	return s.next.Cancel(ctx, req)
}

func (s *tracingMiddleware) GetByLandlordID(ctx context.Context, req *api.GetByLandlordIDRequest) (
	*api.RtosResponse, error) {
	span, ctx := opentracing.StartSpanFromContext(
		ctx, s.middlewareAbleName+" GetByLandlordID")
	defer span.Finish()

	return s.next.GetByLandlordID(ctx, req)
}

func (s *tracingMiddleware) GetByPropertyID(ctx context.Context, req *api.GetByPropertyIDRequest) (
	*api.RtoResponse, error) {
	span, ctx := opentracing.StartSpanFromContext(
		ctx, s.middlewareAbleName+" GetByPropertyID")
	defer span.Finish()

	return s.next.GetByPropertyID(ctx, req)
}

func (s *tracingMiddleware) GetByPropertyIDs(ctx context.Context, req *api.GetByPropertyIDsRequest) (
	*api.RtosResponse, error) {
	span, ctx := opentracing.StartSpanFromContext(
		ctx, s.middlewareAbleName+" GetByPropertyIDs")
	defer span.Finish()

	return s.next.GetByPropertyIDs(ctx, req)
}

func (s *tracingMiddleware) AnswerFromTenant(ctx context.Context, req *api.AnswerFromTenantRequest) (
	*api.RtoResponse, error) {
	span, ctx := opentracing.StartSpanFromContext(
		ctx, s.middlewareAbleName+" AnswerFromTenant")
	defer span.Finish()

	return s.next.AnswerFromTenant(ctx, req)
}

func (s *tracingMiddleware) RentPayment(ctx context.Context, req *api.RentPaymentRequest) (
	*api.RtoResponse, error) {
	span, ctx := opentracing.StartSpanFromContext(
		ctx, s.middlewareAbleName+" RentPayment")
	defer span.Finish()

	return s.next.RentPayment(ctx, req)
}

func (s *tracingMiddleware) LandlordApproved(ctx context.Context, req *api.LandlordApprovedRequest) (
	*api.IdsResponse, error) {
	span, ctx := opentracing.StartSpanFromContext(
		ctx, s.middlewareAbleName+" LandlordApproved")
	defer span.Finish()

	return s.next.LandlordApproved(ctx, req)
}

func (s *tracingMiddleware) UpdateLeadTenant(ctx context.Context, req *api.UpdateLeadTenantRequest) (
	*api.UpdateLeadTenantResponse, error) {
	span, ctx := opentracing.StartSpanFromContext(
		ctx, s.middlewareAbleName+" UpdateLeadTenant")
	defer span.Finish()

	return s.next.UpdateLeadTenant(ctx, req)
}

func (s *tracingMiddleware) UpdateLeadTenantUser(ctx context.Context, req *api.UpdateLeadTenantUserRequest) (
	*api.RtoResponse, error) {
	span, ctx := opentracing.StartSpanFromContext(
		ctx, s.middlewareAbleName+" UpdateLeadTenantUser")
	defer span.Finish()

	return s.next.UpdateLeadTenantUser(ctx, req)
}
