package answerfromtenant

import (
	"context"

	request "gitlab.mashplace.com/mashplace/backend/offer/internal/app/domain/rto/resource/request/usecase"
	"gitlab.mashplace.com/mashplace/backend/offer/internal/app/domain/rto/resource/response"
	"gitlab.mashplace.com/mashplace/backend/offer/internal/app/usecase/rto"

	"github.com/opentracing/opentracing-go"
)

type tracingMiddleware struct {
	next               rto.AnswerFromTenantUseCase
	middlewareAbleName string
}

func NewTracingMiddleware(s rto.AnswerFromTenantUseCase,
	middlewareAbleName string) rto.AnswerFromTenantUseCase {
	return &tracingMiddleware{
		next:               s,
		middlewareAbleName: middlewareAbleName,
	}
}

func (s *tracingMiddleware) AnswerFromTenant(ctx context.Context, req *request.AnswerFromTenantRequest) (
	*response.RtoResponse, error) {
	span, ctx := opentracing.StartSpanFromContext(
		ctx, s.middlewareAbleName+" AnswerFromTenant")
	defer span.Finish()

	return s.next.AnswerFromTenant(ctx, req)
}
