package create

import (
	"context"

	request "gitlab.mashplace.com/mashplace/backend/offer/internal/app/domain/rto/resource/request/usecase"
	"gitlab.mashplace.com/mashplace/backend/offer/internal/app/domain/rto/resource/response"
	"gitlab.mashplace.com/mashplace/backend/offer/internal/app/usecase/rto"

	"github.com/opentracing/opentracing-go"
)

type tracingMiddleware struct {
	next               rto.CreateUseCase
	middlewareAbleName string
}

func NewTracingMiddleware(s rto.CreateUseCase,
	middlewareAbleName string) rto.CreateUseCase {
	return &tracingMiddleware{
		next:               s,
		middlewareAbleName: middlewareAbleName,
	}
}

func (s *tracingMiddleware) Create(ctx context.Context, req *request.CreateRequest) (*response.RtoResponse, error) {
	span, ctx := opentracing.StartSpanFromContext(
		ctx, s.middlewareAbleName+" Create")
	defer span.Finish()

	return s.next.Create(ctx, req)
}
