package createbyguest

import (
	"context"

	request "gitlab.mashplace.com/mashplace/backend/offer/internal/app/domain/rto/resource/request/usecase"
	"gitlab.mashplace.com/mashplace/backend/offer/internal/app/domain/rto/resource/response"
	"gitlab.mashplace.com/mashplace/backend/offer/internal/app/usecase/rto"

	"github.com/opentracing/opentracing-go"
)

type tracingMiddleware struct {
	next               rto.CreateByGuestUseCase
	middlewareAbleName string
}

func NewTracingMiddleware(s rto.CreateByGuestUseCase,
	middlewareAbleName string) rto.CreateByGuestUseCase {
	return &tracingMiddleware{
		next:               s,
		middlewareAbleName: middlewareAbleName,
	}
}

func (s *tracingMiddleware) CreateByGuest(ctx context.Context, req *request.CreateByGuestRequest) (
	*response.RtoResponse, error) {
	span, ctx := opentracing.StartSpanFromContext(
		ctx, s.middlewareAbleName+" CreateByGuest")
	defer span.Finish()

	return s.next.CreateByGuest(ctx, req)
}
