package createtenancy

import (
	"context"

	request "gitlab.mashplace.com/mashplace/backend/offer/internal/app/domain/rto/resource/request/usecase"
	"gitlab.mashplace.com/mashplace/backend/offer/internal/app/domain/rto/resource/response"
	"gitlab.mashplace.com/mashplace/backend/offer/internal/app/usecase/rto"

	"github.com/opentracing/opentracing-go"
)

type tracingMiddleware struct {
	next               rto.CreateTenancyUseCase
	middlewareAbleName string
}

func NewTracingMiddleware(s rto.CreateTenancyUseCase,
	middlewareAbleName string) rto.CreateTenancyUseCase {
	return &tracingMiddleware{
		next:               s,
		middlewareAbleName: middlewareAbleName,
	}
}

func (s *tracingMiddleware) CreateTenancy(ctx context.Context, req *request.CreateTenancyRequest) (
	*response.RtoResponse, error) {
	span, ctx := opentracing.StartSpanFromContext(
		ctx, s.middlewareAbleName+" CreateTenancy")
	defer span.Finish()

	return s.next.CreateTenancy(ctx, req)
}
