package getbylandlordid

import (
	"context"

	request "gitlab.mashplace.com/mashplace/backend/offer/internal/app/domain/rto/resource/request/usecase"
	"gitlab.mashplace.com/mashplace/backend/offer/internal/app/domain/rto/resource/response"
	"gitlab.mashplace.com/mashplace/backend/offer/internal/app/usecase/rto"

	"github.com/opentracing/opentracing-go"
)

type tracingMiddleware struct {
	next               rto.GetByLandlordIDUseCase
	middlewareAbleName string
}

func NewTracingMiddleware(s rto.GetByLandlordIDUseCase,
	middlewareAbleName string) rto.GetByLandlordIDUseCase {
	return &tracingMiddleware{
		next:               s,
		middlewareAbleName: middlewareAbleName,
	}
}

func (s *tracingMiddleware) GetByLandlordID(ctx context.Context, req *request.GetByLandlordIDRequest) (
	*response.RtosResponse, error) {
	span, ctx := opentracing.StartSpanFromContext(
		ctx, s.middlewareAbleName+" GetByLandlordID")
	defer span.Finish()

	return s.next.GetByLandlordID(ctx, req)
}
