package getbypropertyid

import (
	"context"

	request "gitlab.mashplace.com/mashplace/backend/offer/internal/app/domain/rto/resource/request/usecase"
	"gitlab.mashplace.com/mashplace/backend/offer/internal/app/domain/rto/resource/response"
	"gitlab.mashplace.com/mashplace/backend/offer/internal/app/usecase/rto"

	"github.com/opentracing/opentracing-go"
)

type tracingMiddleware struct {
	next               rto.GetByPropertyIDUseCase
	middlewareAbleName string
}

func NewTracingMiddleware(s rto.GetByPropertyIDUseCase,
	middlewareAbleName string) rto.GetByPropertyIDUseCase {
	return &tracingMiddleware{
		next:               s,
		middlewareAbleName: middlewareAbleName,
	}
}

func (s *tracingMiddleware) GetByPropertyID(ctx context.Context, req *request.GetByPropertyIDRequest) (
	*response.RtoResponse, error) {
	span, ctx := opentracing.StartSpanFromContext(
		ctx, s.middlewareAbleName+" GetByPropertyID")
	defer span.Finish()

	return s.next.GetByPropertyID(ctx, req)
}
