package getbypropertyids

import (
	"context"

	request "gitlab.mashplace.com/mashplace/backend/offer/internal/app/domain/rto/resource/request/usecase"
	"gitlab.mashplace.com/mashplace/backend/offer/internal/app/domain/rto/resource/response"
	"gitlab.mashplace.com/mashplace/backend/offer/internal/app/usecase/rto"

	"github.com/opentracing/opentracing-go"
)

type tracingMiddleware struct {
	next               rto.GetByPropertyIDsUseCase
	middlewareAbleName string
}

func NewTracingMiddleware(s rto.GetByPropertyIDsUseCase,
	middlewareAbleName string) rto.GetByPropertyIDsUseCase {
	return &tracingMiddleware{
		next:               s,
		middlewareAbleName: middlewareAbleName,
	}
}

func (s *tracingMiddleware) GetByPropertyIDs(ctx context.Context, req *request.GetByPropertyIDsRequest) (
	*response.RtosResponse, error) {
	span, ctx := opentracing.StartSpanFromContext(
		ctx, s.middlewareAbleName+" GetByPropertyIDs")
	defer span.Finish()

	return s.next.GetByPropertyIDs(ctx, req)
}
