package landlordapproved

import (
	"context"

	request "gitlab.mashplace.com/mashplace/backend/offer/internal/app/domain/rto/resource/request/usecase"
	"gitlab.mashplace.com/mashplace/backend/offer/internal/app/domain/rto/resource/response"
	"gitlab.mashplace.com/mashplace/backend/offer/internal/app/usecase/rto"

	"github.com/opentracing/opentracing-go"
)

type tracingMiddleware struct {
	next               rto.LandlordApprovedUseCase
	middlewareAbleName string
}

func NewTracingMiddleware(s rto.LandlordApprovedUseCase,
	middlewareAbleName string) rto.LandlordApprovedUseCase {
	return &tracingMiddleware{
		next:               s,
		middlewareAbleName: middlewareAbleName,
	}
}

func (s *tracingMiddleware) LandlordApproved(ctx context.Context, req *request.LandlordApprovedRequest) (
	*response.IDsResponse, error) {
	span, ctx := opentracing.StartSpanFromContext(
		ctx, s.middlewareAbleName+" LandlordApproved")
	defer span.Finish()

	return s.next.LandlordApproved(ctx, req)
}
