package renewal

import (
	"context"

	request "gitlab.mashplace.com/mashplace/backend/offer/internal/app/domain/rto/resource/request/usecase"
	"gitlab.mashplace.com/mashplace/backend/offer/internal/app/domain/rto/resource/response"
	"gitlab.mashplace.com/mashplace/backend/offer/internal/app/usecase/rto"

	"github.com/opentracing/opentracing-go"
)

type tracingMiddleware struct {
	next               rto.RenewalUseCase
	middlewareAbleName string
}

func NewTracingMiddleware(s rto.RenewalUseCase,
	middlewareAbleName string) rto.RenewalUseCase {
	return &tracingMiddleware{
		next:               s,
		middlewareAbleName: middlewareAbleName,
	}
}

func (s *tracingMiddleware) Renewal(ctx context.Context, req *request.RenewalRequest) (*response.RtoResponse, error) {
	span, ctx := opentracing.StartSpanFromContext(
		ctx, s.middlewareAbleName+" Renewal")
	defer span.Finish()

	return s.next.Renewal(ctx, req)
}
