package rentpayment

import (
	"context"

	request "gitlab.mashplace.com/mashplace/backend/offer/internal/app/domain/rto/resource/request/usecase"
	"gitlab.mashplace.com/mashplace/backend/offer/internal/app/domain/rto/resource/response"
	"gitlab.mashplace.com/mashplace/backend/offer/internal/app/usecase/rto"

	"github.com/opentracing/opentracing-go"
)

type tracingMiddleware struct {
	next               rto.RentPaymentUseCase
	middlewareAbleName string
}

func NewTracingMiddleware(s rto.RentPaymentUseCase,
	middlewareAbleName string) rto.RentPaymentUseCase {
	return &tracingMiddleware{
		next:               s,
		middlewareAbleName: middlewareAbleName,
	}
}

func (s *tracingMiddleware) RentPayment(ctx context.Context, req *request.RentPaymentRequest) (
	*response.RtoResponse, error) {
	span, ctx := opentracing.StartSpanFromContext(
		ctx, s.middlewareAbleName+" RentPayment")
	defer span.Finish()

	return s.next.RentPayment(ctx, req)
}
