package standingorder

import (
	"context"

	request "gitlab.mashplace.com/mashplace/backend/offer/internal/app/domain/rto/resource/request/usecase"
	"gitlab.mashplace.com/mashplace/backend/offer/internal/app/domain/rto/resource/response"
	"gitlab.mashplace.com/mashplace/backend/offer/internal/app/usecase/rto"

	"github.com/opentracing/opentracing-go"
)

type tracingMiddleware struct {
	next               rto.StandingOrderUseCase
	middlewareAbleName string
}

func NewTracingMiddleware(s rto.StandingOrderUseCase,
	middlewareAbleName string) rto.StandingOrderUseCase {
	return &tracingMiddleware{
		next:               s,
		middlewareAbleName: middlewareAbleName,
	}
}

func (s *tracingMiddleware) StandingOrder(ctx context.Context, req *request.StandingOrderRequest) (
	*response.RtoResponse, error) {
	span, ctx := opentracing.StartSpanFromContext(
		ctx, s.middlewareAbleName+" StandingOrder")
	defer span.Finish()

	return s.next.StandingOrder(ctx, req)
}
