package updateenddate

import (
	"context"

	request "gitlab.mashplace.com/mashplace/backend/offer/internal/app/domain/rto/resource/request/usecase"
	"gitlab.mashplace.com/mashplace/backend/offer/internal/app/domain/rto/resource/response"
	"gitlab.mashplace.com/mashplace/backend/offer/internal/app/usecase/rto"

	"github.com/opentracing/opentracing-go"
)

type tracingMiddleware struct {
	next               rto.UpdateEndDateUseCase
	middlewareAbleName string
}

func NewTracingMiddleware(s rto.UpdateEndDateUseCase,
	middlewareAbleName string) rto.UpdateEndDateUseCase {
	return &tracingMiddleware{
		next:               s,
		middlewareAbleName: middlewareAbleName,
	}
}

func (s *tracingMiddleware) UpdateEndDate(ctx context.Context, req *request.UpdateEndDateRequest) (
	*response.RtoResponse, error) {
	span, ctx := opentracing.StartSpanFromContext(
		ctx, s.middlewareAbleName+" UpdateEndDate")
	defer span.Finish()

	return s.next.UpdateEndDate(ctx, req)
}
