package updateleadtenant

import (
	"context"

	request "gitlab.mashplace.com/mashplace/backend/offer/internal/app/domain/rto/resource/request/usecase"
	response "gitlab.mashplace.com/mashplace/backend/offer/internal/app/domain/rto/resource/response/usecase"
	"gitlab.mashplace.com/mashplace/backend/offer/internal/app/usecase/rto"

	"github.com/opentracing/opentracing-go"
)

type tracingMiddleware struct {
	next               rto.UpdateLeadTenantUseCase
	middlewareAbleName string
}

func NewTracingMiddleware(s rto.UpdateLeadTenantUseCase,
	middlewareAbleName string) rto.UpdateLeadTenantUseCase {
	return &tracingMiddleware{
		next:               s,
		middlewareAbleName: middlewareAbleName,
	}
}

func (s *tracingMiddleware) UpdateLeadTenant(ctx context.Context, req *request.UpdateLeadTenantRequest) (
	*response.UpdateLeadTenantResponse, error) {
	span, ctx := opentracing.StartSpanFromContext(
		ctx, s.middlewareAbleName+" UpdateLeadTenant")
	defer span.Finish()

	return s.next.UpdateLeadTenant(ctx, req)
}
