package updaterentplan

import (
	"context"

	request "gitlab.mashplace.com/mashplace/backend/offer/internal/app/domain/rto/resource/request/usecase"
	"gitlab.mashplace.com/mashplace/backend/offer/internal/app/domain/rto/resource/response"
	"gitlab.mashplace.com/mashplace/backend/offer/internal/app/usecase/rto"

	"github.com/opentracing/opentracing-go"
)

type tracingMiddleware struct {
	next               rto.UpdateRentPlanUseCase
	middlewareAbleName string
}

func NewTracingMiddleware(s rto.UpdateRentPlanUseCase,
	middlewareAbleName string) rto.UpdateRentPlanUseCase {
	return &tracingMiddleware{
		next:               s,
		middlewareAbleName: middlewareAbleName,
	}
}

func (s *tracingMiddleware) UpdateRentPlan(ctx context.Context, req *request.UpdateRentPlanRequest) (
	*response.RtoResponse, error) {
	span, ctx := opentracing.StartSpanFromContext(
		ctx, s.middlewareAbleName+" UpdateRentPlan")
	defer span.Finish()

	return s.next.UpdateRentPlan(ctx, req)
}
