package updatestatetoactive

import (
	"context"
	"time"

	"gitlab.mashplace.com/mashplace/backend/common/generic/log"
	"gitlab.mashplace.com/mashplace/backend/common/lib/constants"
	loggerservice "gitlab.mashplace.com/mashplace/backend/common/lib/logger/service"
	request "gitlab.mashplace.com/mashplace/backend/offer/internal/app/domain/rto/resource/request/usecase"
	"gitlab.mashplace.com/mashplace/backend/offer/internal/app/domain/rto/resource/response"
	"gitlab.mashplace.com/mashplace/backend/offer/internal/app/usecase/rto"

	"github.com/davecgh/go-spew/spew"
)

type loggingMiddleware struct {
	next          rto.UpdateStateToActiveUseCase
	interfaceName string
	logger        loggerservice.Logger
}

func NewLoggingMiddleware(s rto.UpdateStateToActiveUseCase,
	interfaceName string, logger loggerservice.Logger) rto.UpdateStateToActiveUseCase {
	return &loggingMiddleware{
		next:          s,
		interfaceName: interfaceName,
		logger:        logger,
	}
}

func (s *loggingMiddleware) UpdateStateToActive(ctx context.Context, req *request.UpdateStateToActiveRequest) (
	resp *response.RtoResponse, err error) {
	defer func(begin time.Time) {
		log.WithContext(ctx).WithFields(log.Fields{
			"method":              s.interfaceName + "_" + "UpdateStateToActive",
			"req":                 spew.Sdump(req),
			"resp":                spew.Sdump(resp),
			"took":                time.Since(begin),
			constants.LogErrorKey: err,
		}).Debug()
	}(time.Now())

	return s.next.UpdateStateToActive(ctx, req)
}
