package updatestatetoactive

import (
	"context"

	request "gitlab.mashplace.com/mashplace/backend/offer/internal/app/domain/rto/resource/request/usecase"
	"gitlab.mashplace.com/mashplace/backend/offer/internal/app/domain/rto/resource/response"
	"gitlab.mashplace.com/mashplace/backend/offer/internal/app/usecase/rto"

	"github.com/opentracing/opentracing-go"
)

type tracingMiddleware struct {
	next               rto.UpdateStateToActiveUseCase
	middlewareAbleName string
}

func NewTracingMiddleware(s rto.UpdateStateToActiveUseCase,
	middlewareAbleName string) rto.UpdateStateToActiveUseCase {
	return &tracingMiddleware{
		next:               s,
		middlewareAbleName: middlewareAbleName,
	}
}

func (s *tracingMiddleware) UpdateStateToActive(ctx context.Context, req *request.UpdateStateToActiveRequest) (
	*response.RtoResponse, error) {
	span, ctx := opentracing.StartSpanFromContext(
		ctx, s.middlewareAbleName+" UpdateStateToActive")
	defer span.Finish()

	return s.next.UpdateStateToActive(ctx, req)
}
