package updatestatetoending

import (
	"context"

	request "gitlab.mashplace.com/mashplace/backend/offer/internal/app/domain/rto/resource/request/usecase"
	"gitlab.mashplace.com/mashplace/backend/offer/internal/app/domain/rto/resource/response"
	"gitlab.mashplace.com/mashplace/backend/offer/internal/app/usecase/rto"

	"github.com/opentracing/opentracing-go"
)

type tracingMiddleware struct {
	next               rto.UpdateStateToEndingUseCase
	middlewareAbleName string
}

func NewTracingMiddleware(s rto.UpdateStateToEndingUseCase,
	middlewareAbleName string) rto.UpdateStateToEndingUseCase {
	return &tracingMiddleware{
		next:               s,
		middlewareAbleName: middlewareAbleName,
	}
}

func (s *tracingMiddleware) UpdateStateToEnding(ctx context.Context, req *request.UpdateStateToEndingRequest) (
	*response.RtoResponse, error) {
	span, ctx := opentracing.StartSpanFromContext(
		ctx, s.middlewareAbleName+" UpdateStateToEnding")
	defer span.Finish()

	return s.next.UpdateStateToEnding(ctx, req)
}
