package updatestatetofinished

import (
	"context"

	request "gitlab.mashplace.com/mashplace/backend/offer/internal/app/domain/rto/resource/request/usecase"
	"gitlab.mashplace.com/mashplace/backend/offer/internal/app/domain/rto/resource/response"
	"gitlab.mashplace.com/mashplace/backend/offer/internal/app/usecase/rto"

	"github.com/opentracing/opentracing-go"
)

type tracingMiddleware struct {
	next               rto.UpdateStateToFinishedUseCase
	middlewareAbleName string
}

func NewTracingMiddleware(s rto.UpdateStateToFinishedUseCase,
	middlewareAbleName string) rto.UpdateStateToFinishedUseCase {
	return &tracingMiddleware{
		next:               s,
		middlewareAbleName: middlewareAbleName,
	}
}

func (s *tracingMiddleware) UpdateStateToFinished(ctx context.Context, req *request.UpdateStateToFinishedRequest) (
	*response.RtoResponse, error) {
	span, ctx := opentracing.StartSpanFromContext(
		ctx, s.middlewareAbleName+" UpdateStateToFinished")
	defer span.Finish()

	return s.next.UpdateStateToFinished(ctx, req)
}
