package updatestatetowaitingforpayment

import (
	"context"

	request "gitlab.mashplace.com/mashplace/backend/offer/internal/app/domain/rto/resource/request/usecase"
	"gitlab.mashplace.com/mashplace/backend/offer/internal/app/domain/rto/resource/response"
	"gitlab.mashplace.com/mashplace/backend/offer/internal/app/usecase/rto"

	"github.com/opentracing/opentracing-go"
)

type tracingMiddleware struct {
	next               rto.UpdateStateToWaitingForPaymentUseCase
	middlewareAbleName string
}

func NewTracingMiddleware(s rto.UpdateStateToWaitingForPaymentUseCase,
	middlewareAbleName string) rto.UpdateStateToWaitingForPaymentUseCase {
	return &tracingMiddleware{
		next:               s,
		middlewareAbleName: middlewareAbleName,
	}
}

func (s *tracingMiddleware) UpdateStateToWaitingForPayment(ctx context.Context, req *request.UpdateStateToWaitingForPaymentRequest) (
	*response.RtoResponse, error) {
	span, ctx := opentracing.StartSpanFromContext(
		ctx, s.middlewareAbleName+" UpdateStateToWaitingForPayment")
	defer span.Finish()

	return s.next.UpdateStateToWaitingForPayment(ctx, req)
}
