package service_test

import (
	"errors"
	"strings"
	"testing"

	account "gitlab.mashplace.com/mashplace/backend/account/api"
	mockaccount "gitlab.mashplace.com/mashplace/backend/account/api/mock"
	request "gitlab.mashplace.com/mashplace/backend/offer/internal/app/domain/rto/resource/request/service"
	"gitlab.mashplace.com/mashplace/backend/offer/internal/app/service"

	"github.com/golang/mock/gomock"
	. "github.com/smartystreets/goconvey/convey"
)

func TestAccountService(t *testing.T) {
	Convey("SecurityService", t, func() {
		ctrl := gomock.NewController(t)
		defer ctrl.Finish()

		accountServiceClient := mockaccount.NewMockAccountServiceClient(ctrl)
		srv := service.NewAccountService(accountServiceClient)
		So(srv, ShouldImplement, (*service.AccountService)(nil))

		Convey("IsVeriffValid", func() {
			req := &request.IsVeriffValidRequest{}
			err := srv.IsVeriffValid(ctx, req)
			So(err, ShouldBeError)
			So(strings.Contains(err.Error(), service.IsVeriffValidNotValidRequestErrorText), ShouldBeTrue)

			testError := errors.New("mambeti - error")
			req.LandlordID = "kantar-oldi-mali"
			accountServiceClient.EXPECT().IsVeriffValid(ctx, &account.IsVeriffValidRequest{
				AccountId: req.LandlordID,
			}).Return(nil, testError)
			err = srv.IsVeriffValid(ctx, req)
			So(err, ShouldBeError)
			So(strings.Contains(err.Error(), service.IsVeriffValidErrorText), ShouldBeTrue)

			accountServiceClient.EXPECT().IsVeriffValid(ctx, &account.IsVeriffValidRequest{
				AccountId: req.LandlordID,
			}).Return(&account.IsVeriffValidResponse{
				Success: false,
			}, nil)
			err = srv.IsVeriffValid(ctx, req)
			So(err, ShouldBeError)
			So(strings.Contains(err.Error(), service.IsVeriffValidNotSuccessErrorText), ShouldBeTrue)

			accountServiceClient.EXPECT().IsVeriffValid(ctx, &account.IsVeriffValidRequest{
				AccountId: req.LandlordID,
			}).Return(&account.IsVeriffValidResponse{
				Success: true,
			}, nil)
			err = srv.IsVeriffValid(ctx, req)
			So(err, ShouldBeNil)
		})
	})
}
