package service

import (
	"context"

	extras "gitlab.mashplace.com/mashplace/backend/extras/api"
	invoice "gitlab.mashplace.com/mashplace/backend/invoice/api"
	request "gitlab.mashplace.com/mashplace/backend/offer/internal/app/domain/rto/resource/request/service"
	"gitlab.mashplace.com/mashplace/backend/offer/internal/connections"

	"github.com/pkg/errors"
)

type InvoiceService interface {
	CreateStandingOrderInvoice(ctx context.Context, req *request.CreateStandingOrderInvoiceRequest) (string, error)
}

type invoiceService struct {
	client invoice.InvoiceServiceClient
}

func NewInvoiceService(invoiceServiceClient invoice.InvoiceServiceClient) InvoiceService {
	return &invoiceService{
		client: invoiceServiceClient,
	}
}

func (is *invoiceService) CreateStandingOrderInvoice(ctx context.Context,
	req *request.CreateStandingOrderInvoiceRequest) (string, error) {
	err := req.Validate()
	if err != nil {
		return "", errors.Wrap(err, "invoiceService req.Validate")
	}

	resp, err := is.client.InvoiceCreate(ctx, &invoice.InvoiceCreateRequest{
		CorrelationType:   invoice.CorrelationTypeOffer,
		CorrelationId:     req.Rto.ID.Hex(),
		CorrelationTarget: invoice.CorrelationTargetStandingOrder,
		Periodic:          true,
		Property:          req.Rto.Property,
		Ivr:               req.Rto.PropertyIvr,
		Listing:           req.Rto.PropertyListing,
		User:              req.Rto.LeadTenant.User,
		Services:          []*extras.ServiceSnapshot{req.ServiceSnapshot},
		NotifyCallback:    connections.SelfRemote,
	})
	if err != nil {
		return "", errors.Wrap(err, "invoiceService client.InvoiceCreate")
	}

	return resp.GetInvoice().GetInvoiceId(), nil
}
