package service_test

import (
	"testing"

	listing "gitlab.mashplace.com/mashplace/backend/listing/api"
	mocklisting "gitlab.mashplace.com/mashplace/backend/listing/api/mock"
	"gitlab.mashplace.com/mashplace/backend/offer/internal/app/service"

	"github.com/golang/mock/gomock"
	. "github.com/smartystreets/goconvey/convey"
)

func TestListingService(t *testing.T) {
	Convey("ListingService", t, func() {
		ctrl := gomock.NewController(t)
		defer ctrl.Finish()

		listingServiceClient := mocklisting.NewMockListingServiceClient(ctrl)
		srv := service.NewListingService(listingServiceClient)
		So(srv, ShouldImplement, (*service.ListingService)(nil))

		Convey("VersionHasNotEmptyBankDetails", func() {
			bankDetails := &listing.BankDetails{
				SortCode:          "chmoshiebanniebarligikimmagantuisiemesolsin",
				AccountNumber:     "mopedshikchertpojiznigondon",
				PersonalReference: "kirgizskieshlyuhikonchennieekikesarevooledi",
			}
			listingVersion := &listing.Version{
				Rental: &listing.Rental{BankDetails: bankDetails},
			}
			So(srv.VersionHasNotEmptyBankDetails(ctx, listingVersion), ShouldBeFalse)

			bankDetails.AccountName = "orisotstrahachutneumerchertuahaha"
			So(srv.VersionHasNotEmptyBankDetails(ctx, listingVersion), ShouldBeTrue)
		})
	})
}
