package rto_test

import (
	"testing"
	"time"

	"gitlab.mashplace.com/mashplace/backend/common/lib/httpclient/fasthttpclient"
	"gitlab.mashplace.com/mashplace/backend/common/lib/marshallunmarshall/jsoniterlocal"
	api "gitlab.mashplace.com/mashplace/backend/offer/api/rto"
	domain "gitlab.mashplace.com/mashplace/backend/offer/internal/app/domain/rto"
	request "gitlab.mashplace.com/mashplace/backend/offer/internal/app/domain/rto/resource/request/service/rto"
	"gitlab.mashplace.com/mashplace/backend/offer/internal/app/service/rto"
	"gitlab.mashplace.com/mashplace/backend/offer/test"

	. "github.com/smartystreets/goconvey/convey"
	"go.mongodb.org/mongo-driver/bson/primitive"
)

func TestCloseComService(t *testing.T) {
	if test.CanRunIntegrationalTests() {
		Convey("CloseComService", t, func() {
			httpClient := fasthttpclient.NewHTTPClient(5, 5)
			marshallUnmarshall := jsoniterlocal.NewMarshalUnmarshal()
			srv := rto.NewCloseComService(httpClient, marshallUnmarshall)
			So(srv, ShouldImplement, (*rto.CloseComService)(nil))

			now := time.Now()
			rtoModel := &domain.Rto{
				Conditions: &domain.Conditions{
					MoveInDate: now,
					EndDate:    now,
					Price:      23.0,
				},
				LeadTenant: &domain.Tenant{
					User:         test.KorshilerDogs,
					EmailPrimary: test.ZhunusoviDogs,
				},
				Landlord: &domain.Landlord{
					User:      primitive.NewObjectID().Hex(),
					FirstName: test.KorshilerDogs,
					LastName:  "barligikimmagantuisiemesolsin",
				},
				PropertyAddress: test.ZhunusoviDogs,
				Property:        test.KorshilerDogs,
				RentPlan:        api.RentPlan_RENT_PLAN_ESSENTIAL,
				State:           api.RtoState_RTO_STATE_WAITING_FOR_TENANT_CONFIRMATION,
				ID:              primitive.NewObjectID(),
			}
			req := &request.CloseComSaveOpportunityRequest{
				Rto:                  rtoModel,
				RtoPreviousState:     api.RtoState_RTO_STATE_PENDING_SETUP,
				RtoPreviousStateDate: now,
			}
			saveOpportunityResponse, err := srv.SaveOpportunity(ctx, req)
			So(err, ShouldBeNil)
			So(saveOpportunityResponse.OpportunityID, ShouldNotBeEmpty)
			So(saveOpportunityResponse.LeadID, ShouldNotBeEmpty)

			time.Sleep(3 * time.Second)

			req.Rto.OpportunityID = saveOpportunityResponse.OpportunityID
			if saveOpportunityResponse.LeadID != "" {
				req.Rto.Landlord.LeadID = saveOpportunityResponse.LeadID
			}
			req.RtoPreviousState = api.RtoState_RTO_STATE_CANCELLED
			saveOpportunityResponse, err = srv.SaveOpportunity(ctx, req)
			So(err, ShouldBeNil)
			So(saveOpportunityResponse.OpportunityID, ShouldNotBeEmpty)
			So(saveOpportunityResponse.LeadID, ShouldBeEmpty)

			err = srv.DeleteOpportunity(ctx, saveOpportunityResponse.OpportunityID)
			So(err, ShouldBeNil)

			closeComFindLeadRequest := request.NewCloseComFindLeadRequest(rtoModel.Landlord.User)
			leadID, err := srv.FindLead(ctx, closeComFindLeadRequest)
			So(err, ShouldBeNil)
			So(leadID, ShouldNotBeEmpty)

			lead, err := srv.GetLead(ctx, leadID)
			So(err, ShouldBeNil)
			So(lead, ShouldNotBeEmpty)
			So(lead.ID, ShouldEqual, leadID)
			So(lead.Name, ShouldEqual, rtoModel.Landlord.GetFullName())

			err = srv.DeleteLead(ctx, leadID)
			So(err, ShouldBeNil)

			time.Sleep(3 * time.Second)

			leadID, err = srv.FindLead(ctx, closeComFindLeadRequest)
			So(err, ShouldBeNil)
			So(leadID, ShouldBeEmpty)

			lead, err = srv.GetLead(ctx, leadID)
			So(err, ShouldBeNil)
			So(lead.ID, ShouldBeEmpty)
		})
	}
}
