package daemon_test

import (
	"testing"
	"time"

	api "gitlab.mashplace.com/mashplace/backend/offer/api/rto"
	"gitlab.mashplace.com/mashplace/backend/offer/internal/app/service/rto/daemon"
	usecase "gitlab.mashplace.com/mashplace/backend/offer/internal/app/usecase/rto"
	"gitlab.mashplace.com/mashplace/backend/offer/test"

	. "github.com/smartystreets/goconvey/convey"
)

func TestFinishService(t *testing.T) {
	if test.CanRunTests() {
		Convey("FinishService", t, func() {
			updateStateToFinishedUseCase := usecase.NewUpdateStateToFinishedUseCase(
				repository, historyRepository, closeComService)
			srv := daemon.NewFinishService(repository, updateStateToFinishedUseCase)
			So(srv, ShouldImplement, (*daemon.FinishService)(nil))

			Convey("Finish", func() {
				res, err := srv.Finish(ctx)
				So(err, ShouldBeNil)
				So(res.IDs, ShouldBeEmpty)

				rtoModel := test.CreateRto(ctx, t, repository, api.RtoState_RTO_STATE_PENDING_SETUP,
					test.KorshilerDogs, test.Chmolgani)

				res, err = srv.Finish(ctx)
				So(err, ShouldBeNil)
				So(res.IDs, ShouldBeEmpty)

				endDate := time.Now().Add(time.Duration(-2) * time.Hour)
				rtoModel.Conditions.EndDate = endDate
				rtoModel.State = api.RtoState_RTO_STATE_ACTIVE
				So(repository.MustUpdate(ctx, rtoModel), ShouldBeNil)

				res, err = srv.Finish(ctx)
				So(err, ShouldBeNil)
				So(len(res.IDs), ShouldEqual, 1)
				So(res.IDs[0], ShouldEqual, rtoModel.ID.Hex())

				res, err = srv.Finish(ctx)
				So(err, ShouldBeNil)
				So(res.IDs, ShouldBeEmpty)

				test.DeleteRto(ctx, t, repository, historyRepository, rtoModel.ID, 1)

				rtoModelActive := test.CreateRto(ctx, t, repository, api.RtoState_RTO_STATE_ACTIVE,
					test.KorshilerDogs, test.Chmolgani)
				rtoModelActive.Conditions.EndDate = endDate
				So(repository.MustUpdate(ctx, rtoModelActive), ShouldBeNil)
				rtoModelActiveRollingTenancy := test.CreateRto(ctx, t, repository,
					api.RtoState_RTO_STATE_ACTIVE_ROLLING_TENANCY, test.KorshilerDogs, test.Chmolgani)
				rtoModelActiveRollingTenancy.Conditions.EndDate = endDate
				So(repository.MustUpdate(ctx, rtoModelActiveRollingTenancy), ShouldBeNil)
				rtoModelEnding := test.CreateRto(ctx, t, repository,
					api.RtoState_RTO_STATE_ENDING, test.KorshilerDogs, test.Chmolgani)
				rtoModelEnding.Conditions.EndDate = endDate
				So(repository.MustUpdate(ctx, rtoModelEnding), ShouldBeNil)

				res, err = srv.Finish(ctx)
				So(err, ShouldBeNil)
				So(len(res.IDs), ShouldEqual, 3)

				res, err = srv.Finish(ctx)
				So(err, ShouldBeNil)
				So(res.IDs, ShouldBeEmpty)

				test.DeleteRto(ctx, t, repository, historyRepository, rtoModelActive.ID, 1)
				test.DeleteRto(ctx, t, repository, historyRepository, rtoModelActiveRollingTenancy.ID, 1)
				test.DeleteRto(ctx, t, repository, historyRepository, rtoModelEnding.ID, 1)
			})
		})
	}
}
