package rto_test

import (
	"testing"

	api "gitlab.mashplace.com/mashplace/backend/offer/api/rto"
	requestservice "gitlab.mashplace.com/mashplace/backend/offer/internal/app/domain/rto/resource/request/service"
	request "gitlab.mashplace.com/mashplace/backend/offer/internal/app/domain/rto/resource/request/service/rto"
	mockservice "gitlab.mashplace.com/mashplace/backend/offer/internal/app/service/mock"
	"gitlab.mashplace.com/mashplace/backend/offer/internal/app/service/rto"
	"gitlab.mashplace.com/mashplace/backend/offer/test"

	"github.com/golang/mock/gomock"
	. "github.com/smartystreets/goconvey/convey"
)

func TestSecurityService(t *testing.T) {
	if test.CanRunTests() {
		Convey("SecurityService", t, func() {
			ctrl := gomock.NewController(t)
			defer ctrl.Finish()

			accountService := mockservice.NewMockAccountService(ctrl)
			srv := rto.NewSecurityService(repository, accountService)
			So(srv, ShouldImplement, (*rto.SecurityService)(nil))

			Convey("CanLandlordUpdateRto", func() {
				rtoModel := test.CreateRto(ctx, t, repository, api.RtoState_RTO_STATE_PENDING_SETUP,
					test.KorshilerDogs, test.Chmolgani)
				rtoID, landlordID := rtoModel.ID.Hex(), rtoModel.Landlord.User
				accountService.EXPECT().IsVeriffValid(ctx, &requestservice.IsVeriffValidRequest{
					LandlordID: landlordID,
				}).Return(nil)
				res, err := srv.CanLandlordUpdateRto(ctx, &request.CanLandlordUpdateRtoRequest{
					ID:              rtoID,
					LandlordID:      landlordID,
					NeedCheckVeriff: true,
				})
				So(err, ShouldBeNil)
				So(res, ShouldNotBeNil)

				Convey("CanUserReadRto", func() {
					canUserReadRtoRequest := &request.CanUserReadRtoRequest{
						UserID: "chmoshibarligikimmagantuisiemesolsin",
						Rto:    rtoModel,
					}
					So(srv.CanUserReadRto(ctx, canUserReadRtoRequest), ShouldBeError)

					canUserReadRtoRequest.UserID = landlordID
					So(srv.CanUserReadRto(ctx, canUserReadRtoRequest), ShouldBeNil)

					test.DeleteRto(ctx, t, repository, historyRepository, rtoModel.ID, 0)
				})
			})
		})
	}
}
