package rto_test

import (
	"strings"
	"testing"

	listing "gitlab.mashplace.com/mashplace/backend/listing/api"
	api "gitlab.mashplace.com/mashplace/backend/offer/api/rto"
	servicerequest "gitlab.mashplace.com/mashplace/backend/offer/internal/app/domain/rto/resource/request/service"
	request "gitlab.mashplace.com/mashplace/backend/offer/internal/app/domain/rto/resource/request/usecase"
	mockappservice "gitlab.mashplace.com/mashplace/backend/offer/internal/app/service/mock"
	"gitlab.mashplace.com/mashplace/backend/offer/internal/app/usecase/rto"
	"gitlab.mashplace.com/mashplace/backend/offer/test"

	"github.com/golang/mock/gomock"
	. "github.com/smartystreets/goconvey/convey"
)

func TestCreateByGuestUseCase(t *testing.T) {
	if test.CanRunTests() {
		Convey("CreateByGuestUseCase", t, func() {
			ctrl := gomock.NewController(t)
			defer ctrl.Finish()

			listingService := mockappservice.NewMockListingService(ctrl)
			useCase := rto.NewCreateByGuestUseCase(repository, historyRepository, listingService)

			propertyID := "barligikimmagantuisiemesolsin"
			listingService.EXPECT().GetVersion(ctx, &servicerequest.ListingGetVersionRequest{
				PropertyID: propertyID,
			}).Return(&listing.Version{PropertyId: propertyID}, nil)
			req := &request.CreateByGuestRequest{
				RentPlan:   api.RentPlan_RENT_PLAN_ESSENTIAL,
				PropertyID: propertyID,
			}
			resp, err := useCase.CreateByGuest(ctx, req)
			So(err, ShouldBeNil)
			So(resp, ShouldNotBeNil)
			rtoModel := resp.Rto

			listingService.EXPECT().GetVersion(ctx, &servicerequest.ListingGetVersionRequest{
				PropertyID: propertyID,
			}).Return(&listing.Version{PropertyId: propertyID}, nil)
			resp, err = useCase.CreateByGuest(ctx, req)
			So(resp, ShouldBeNil)
			So(err, ShouldBeError)
			So(strings.Contains(err.Error(), rto.WithThisPropertyIDRtoAlreadyExistsErrorText), ShouldBeTrue)

			test.DeleteRto(ctx, t, repository, historyRepository, rtoModel.ID, 1)
		})
	}
}
