package rto_test

import (
	"testing"
	"time"

	api "gitlab.mashplace.com/mashplace/backend/offer/api/rto"
	requestservrto "gitlab.mashplace.com/mashplace/backend/offer/internal/app/domain/rto/resource/request/service/rto"
	request "gitlab.mashplace.com/mashplace/backend/offer/internal/app/domain/rto/resource/request/usecase"
	"gitlab.mashplace.com/mashplace/backend/offer/internal/app/domain/rto/resource/response"
	mockappservice "gitlab.mashplace.com/mashplace/backend/offer/internal/app/service/mock"
	mockservice "gitlab.mashplace.com/mashplace/backend/offer/internal/app/service/rto/mock"
	"gitlab.mashplace.com/mashplace/backend/offer/internal/app/usecase/rto"
	"gitlab.mashplace.com/mashplace/backend/offer/test"

	"github.com/golang/mock/gomock"
	. "github.com/smartystreets/goconvey/convey"
)

func TestCreateTenancyUseCase(t *testing.T) {
	if test.CanRunTests() {
		Convey("CreateTenancyUseCase", t, func() {
			ctrl := gomock.NewController(t)
			defer ctrl.Finish()

			accountService := mockappservice.NewMockAccountService(ctrl)
			securityService := mockservice.NewMockSecurityService(ctrl)
			useCase := rto.NewCreateTenancyUseCase(repository, historyRepository, securityService, accountService)

			rtoModel := test.CreateRto(ctx, t, repository, api.RtoState_RTO_STATE_PENDING_SETUP,
				test.ZhunusoviDogs, test.RentAmount)
			rtoID, landlordID := rtoModel.ID.Hex(), rtoModel.Landlord.User

			location, err := time.LoadLocation("Asia/Almaty")
			So(err, ShouldBeNil)

			securityService.EXPECT().CanLandlordUpdateRto(ctx, &requestservrto.CanLandlordUpdateRtoRequest{
				ID:         rtoID,
				LandlordID: landlordID,
			}).Return(&response.RtoResponse{Rto: rtoModel}, nil)
			accountService.EXPECT().GetAccountByEmail(ctx, test.ZhunusoviDogs).Return(nil, nil)
			createTenancyResponse, err := useCase.CreateTenancy(ctx,
				&request.CreateTenancyRequest{
					ID:         rtoID,
					LandlordID: landlordID,
					PaymentData: &request.PaymentData{
						StartDate:  time.Date(1987, 7, 17, 6, 6, 6, 0, location),
						EndDate:    time.Date(1999, 4, 21, 6, 6, 6, 0, location),
						RentAmount: test.RentAmount,
						PaymentDay: 17,
					},
					LeadTenant: &request.User{
						FirstName: "aset-0-anastasiyatrans",
						LastName:  "kulyash-0-koreecsdelalvestrans",
						Email:     test.ZhunusoviDogs,
					},
				})
			So(err, ShouldBeNil)
			So(createTenancyResponse, ShouldNotBeNil)

			test.DeleteRto(ctx, t, repository, historyRepository, rtoModel.ID, 1)
		})
	}
}
