package rto_test

import (
	"strings"
	"testing"

	account "gitlab.mashplace.com/mashplace/backend/account/api"
	listing "gitlab.mashplace.com/mashplace/backend/listing/api"
	api "gitlab.mashplace.com/mashplace/backend/offer/api/rto"
	servicerequest "gitlab.mashplace.com/mashplace/backend/offer/internal/app/domain/rto/resource/request/service"
	request "gitlab.mashplace.com/mashplace/backend/offer/internal/app/domain/rto/resource/request/usecase"
	mockappservice "gitlab.mashplace.com/mashplace/backend/offer/internal/app/service/mock"
	"gitlab.mashplace.com/mashplace/backend/offer/internal/app/usecase/rto"
	"gitlab.mashplace.com/mashplace/backend/offer/test"

	"github.com/golang/mock/gomock"
	. "github.com/smartystreets/goconvey/convey"
	"go.mongodb.org/mongo-driver/bson/primitive"
)

func TestCreateseCase(t *testing.T) {
	if test.CanRunTests() {
		Convey("CreateUseCase", t, func() {
			ctrl := gomock.NewController(t)
			defer ctrl.Finish()

			accountService := mockappservice.NewMockAccountService(ctrl)
			listingService := mockappservice.NewMockListingService(ctrl)
			useCase := rto.NewCreateUseCase(repository,
				historyRepository,
				accountService,
				listingService,
				closeComService)

			landlordID, propertyID := primitive.NewObjectID().Hex(), test.ZhunusoviDogs
			listingService.EXPECT().GetVersion(ctx, &servicerequest.ListingGetVersionRequest{
				PropertyID: propertyID,
				LandlordID: landlordID,
			}).Return(&listing.Version{PropertyId: propertyID}, nil)
			accountService.EXPECT().GetAccountByID(ctx, landlordID).Return(&account.Account{
				ViewedVideosIds: []int32{2065, 2032},
			}, nil)
			req := &request.CreateRequest{
				LandlordID: landlordID,
				RentPlan:   api.RentPlan_RENT_PLAN_ESSENTIAL,
				PropertyID: propertyID,
			}
			resp, err := useCase.Create(ctx, req)
			So(err, ShouldBeNil)
			So(resp, ShouldNotBeNil)
			rtoModel := resp.Rto

			err = closeComService.DeleteOpportunity(ctx, resp.Rto.OpportunityID)
			So(err, ShouldBeNil)

			err = closeComService.DeleteLead(ctx, resp.Rto.GetLeadID())
			So(err, ShouldBeNil)

			listingService.EXPECT().GetVersion(ctx, &servicerequest.ListingGetVersionRequest{
				PropertyID: propertyID,
				LandlordID: landlordID,
			}).Return(&listing.Version{PropertyId: propertyID}, nil)
			resp, err = useCase.Create(ctx, req)
			So(resp, ShouldBeNil)
			So(err, ShouldBeError)
			So(strings.Contains(err.Error(), rto.WithThisPropertyIDRtoAlreadyExistsErrorText), ShouldBeTrue)

			test.DeleteRto(ctx, t, repository, historyRepository, rtoModel.ID, 1)
		})
	}
}
