package rto

import (
	"context"

	"github.com/pkg/errors"
	"go.mongodb.org/mongo-driver/mongo"

	domain "gitlab.mashplace.com/mashplace/backend/offer/internal/app/domain/rto"
	request "gitlab.mashplace.com/mashplace/backend/offer/internal/app/domain/rto/resource/request/usecase"
	"gitlab.mashplace.com/mashplace/backend/offer/internal/app/domain/rto/resource/response"
)

type GetByLandlordIDUseCase interface {
	GetByLandlordID(ctx context.Context, req *request.GetByLandlordIDRequest) (*response.RtosResponse, error)
}

type getByLandlordIDUseCase struct {
	repository domain.Repository
}

func NewGetByLandlordIDUseCase(repository domain.Repository) GetByLandlordIDUseCase {
	return &getByLandlordIDUseCase{
		repository: repository,
	}
}

func (gbpiuc *getByLandlordIDUseCase) GetByLandlordID(ctx context.Context, req *request.GetByLandlordIDRequest) (
	*response.RtosResponse, error) {
	err := req.Validate()
	if err != nil {
		return nil, errors.Wrap(err, "rto GetByLandlordIDUseCase req.Validate")
	}

	rtos, err := gbpiuc.repository.FindAllByLandlordID(ctx, &domain.AllByLandlordIDFilter{
		LandlordID: req.LandlordID,
		States:     req.States,
	})
	if err != nil {
		if errors.Is(err, mongo.ErrNoDocuments) {
			return nil, nil
		}

		return nil, errors.Wrap(err, "rto GetByLandlordIDUseCase repository.FindAllByLandlordID")
	}

	return &response.RtosResponse{
		Rtos: rtos,
	}, nil
}
