package rto

import (
	"context"

	api "gitlab.mashplace.com/mashplace/backend/offer/api/rto"
	domain "gitlab.mashplace.com/mashplace/backend/offer/internal/app/domain/rto"
	request "gitlab.mashplace.com/mashplace/backend/offer/internal/app/domain/rto/resource/request/usecase"
	"gitlab.mashplace.com/mashplace/backend/offer/internal/app/domain/rto/resource/response"
	service "gitlab.mashplace.com/mashplace/backend/offer/internal/app/service/rto"

	"github.com/pkg/errors"
)

type GetByPropertyIDsUseCase interface {
	GetByPropertyIDs(ctx context.Context, req *request.GetByPropertyIDsRequest) (*response.RtosResponse, error)
}

type getByPropertyIDsUseCase struct {
	repository      domain.Repository
	securityService service.SecurityService
}

func NewGetByPropertyIDsUseCase(repository domain.Repository,
	securityService service.SecurityService) GetByPropertyIDsUseCase {
	return &getByPropertyIDsUseCase{
		securityService: securityService,
		repository:      repository,
	}
}

func (gbpiuc *getByPropertyIDsUseCase) GetByPropertyIDs(ctx context.Context, req *request.GetByPropertyIDsRequest) (
	*response.RtosResponse, error) {
	err := req.Validate()
	if err != nil {
		return nil, errors.Wrap(err, "rto getByPropertyIDsUseCase req.Validate")
	}

	rtos, err := gbpiuc.repository.FindAllByPropertyIDs(ctx, &domain.AllByPropertyIDsFilter{
		PropertyIDs: req.PropertyIDs,
		MinState:    api.RtoState_RTO_STATE_PENDING_SETUP,
		MaxState:    api.RtoState_RTO_STATE_CANCELLED,
	})
	if err != nil {
		return nil, errors.Wrap(err, "rto getByPropertyIDsUseCase repository.FindAllByPropertyIDs")
	}

	return &response.RtosResponse{
		Rtos: rtos,
	}, nil
}
