package rto_test

import (
	"testing"

	listing "gitlab.mashplace.com/mashplace/backend/listing/api"
	api "gitlab.mashplace.com/mashplace/backend/offer/api/rto"
	servicerequest "gitlab.mashplace.com/mashplace/backend/offer/internal/app/domain/rto/resource/request/service"
	request "gitlab.mashplace.com/mashplace/backend/offer/internal/app/domain/rto/resource/request/usecase"
	"gitlab.mashplace.com/mashplace/backend/offer/internal/app/domain/rto/resource/response"
	mockappservice "gitlab.mashplace.com/mashplace/backend/offer/internal/app/service/mock"
	"gitlab.mashplace.com/mashplace/backend/offer/internal/app/usecase/rto"
	mockusecaserto "gitlab.mashplace.com/mashplace/backend/offer/internal/app/usecase/rto/mock"
	"gitlab.mashplace.com/mashplace/backend/offer/test"

	"github.com/golang/mock/gomock"
	. "github.com/smartystreets/goconvey/convey"
)

func TestLandlordApprovedUseCase(t *testing.T) {
	if test.CanRunTests() {
		Convey("LandlordApprovedUseCase", t, func() {
			ctrl := gomock.NewController(t)
			defer ctrl.Finish()

			listingService := mockappservice.NewMockListingService(ctrl)
			updateStateToWaitingForTenantConfirmationUseCase :=
				mockusecaserto.NewMockUpdateStateToWaitingForTenantConfirmationUseCase(ctrl)
			useCase := rto.NewLandlordApprovedUseCase(repository,
				listingService, updateStateToWaitingForTenantConfirmationUseCase)

			rtoModel := test.CreateRto(ctx, t, repository,
				api.RtoState_RTO_STATE_PENDING_SETUP, test.KorshilerDogs, test.Chmolgani)
			rtoID, landlordID := rtoModel.ID.Hex(), rtoModel.Landlord.User
			bankDetails := &listing.BankDetails{
				SortCode:          "rustamninsheshesishalava",
				AccountNumber:     "rustamninakesipassivniipidor",
				AccountName:       "rustamninbratpassivniipidor",
				PersonalReference: "rustampassivniipidor",
			}
			listingVersion := &listing.Version{Rental: &listing.Rental{
				BankDetails: bankDetails,
			}}
			listingService.EXPECT().GetVersion(ctx, &servicerequest.ListingGetVersionRequest{
				PropertyID: rtoModel.Property,
				LandlordID: rtoModel.GetLandlordID(),
			}).Return(listingVersion, nil)
			listingService.EXPECT().VersionHasNotEmptyBankDetails(ctx, listingVersion).Return(true)
			updateStateToWaitingForTenantConfirmationUseCase.EXPECT().
				UpdateStateToWaitingForTenantConfirmation(ctx,
					gomock.Any()).Return(&response.RtoResponse{Rto: rtoModel}, nil)
			res, err := useCase.LandlordApproved(ctx, &request.LandlordApprovedRequest{
				LandlordID: landlordID,
			})
			So(err, ShouldBeNil)
			So(len(res.IDs), ShouldEqual, 1)
			So(res.IDs[0], ShouldEqual, rtoID)

			test.DeleteRto(ctx, t, repository, historyRepository, rtoModel.ID, 0)
		})
	}
}
