package rto_test

import (
	"testing"

	listing "gitlab.mashplace.com/mashplace/backend/listing/api"
	api "gitlab.mashplace.com/mashplace/backend/offer/api/rto"
	requestservice "gitlab.mashplace.com/mashplace/backend/offer/internal/app/domain/rto/resource/request/service"
	request "gitlab.mashplace.com/mashplace/backend/offer/internal/app/domain/rto/resource/request/usecase"
	"gitlab.mashplace.com/mashplace/backend/offer/internal/app/domain/rto/resource/response"
	mockappservice "gitlab.mashplace.com/mashplace/backend/offer/internal/app/service/mock"
	"gitlab.mashplace.com/mashplace/backend/offer/internal/app/usecase/rto"
	mockusecaserto "gitlab.mashplace.com/mashplace/backend/offer/internal/app/usecase/rto/mock"
	"gitlab.mashplace.com/mashplace/backend/offer/test"

	"github.com/golang/mock/gomock"
	. "github.com/smartystreets/goconvey/convey"
)

func TestRentPaymentUseCase(t *testing.T) {
	if test.CanRunTests() {
		Convey("RentPaymentUseCase", t, func() {
			ctrl := gomock.NewController(t)
			defer ctrl.Finish()

			listingService := mockappservice.NewMockListingService(ctrl)
			accountService := mockappservice.NewMockAccountService(ctrl)
			updateStateToWaitingForTenantConfirmationUseCase :=
				mockusecaserto.NewMockUpdateStateToWaitingForTenantConfirmationUseCase(ctrl)
			useCase := rto.NewRentPaymentUseCase(repository, accountService,
				listingService, closeComService, updateStateToWaitingForTenantConfirmationUseCase)

			rtoModel := test.CreateRto(ctx, t, repository,
				api.RtoState_RTO_STATE_PENDING_SETUP, test.KorshilerDogs, test.Chmolgani)
			rtoID, landlordID := rtoModel.ID.Hex(), rtoModel.Landlord.User
			bankDetails := &listing.BankDetails{
				SortCode:          "chingizdinsheshesishalava",
				AccountNumber:     "chingizdinakesipassivniipidor",
				AccountName:       "korshilerchertiebannie",
				PersonalReference: "chingizpassivniipidor",
			}
			listingVersion := &listing.Version{
				Rental:     &listing.Rental{BankDetails: bankDetails},
				PropertyId: rtoModel.Property,
			}
			listingService.EXPECT().VersionHasNotEmptyBankDetails(ctx, listingVersion).Return(true)
			accountService.EXPECT().IsVeriffValid(ctx, &requestservice.IsVeriffValidRequest{
				LandlordID: landlordID,
			}).Return(nil)
			updateStateToWaitingForTenantConfirmationUseCase.EXPECT().
				UpdateStateToWaitingForTenantConfirmation(ctx,
					gomock.Any()).Return(&response.RtoResponse{Rto: rtoModel}, nil)
			res, err := useCase.RentPayment(ctx, &request.RentPaymentRequest{
				ListingVersion: listingVersion,
			})
			So(err, ShouldBeNil)
			So(res.Rto.ID.Hex(), ShouldEqual, rtoID)

			test.DeleteRto(ctx, t, repository, historyRepository, rtoModel.ID, 0)
		})
	}
}
