package rto_test

import (
	"strings"
	"testing"

	. "github.com/smartystreets/goconvey/convey"
	api "gitlab.mashplace.com/mashplace/backend/offer/api/rto"
	request "gitlab.mashplace.com/mashplace/backend/offer/internal/app/domain/rto/resource/request/usecase"
	"gitlab.mashplace.com/mashplace/backend/offer/internal/app/usecase/rto"
	"gitlab.mashplace.com/mashplace/backend/offer/test"
)

func TestStandingOrderUseCase(t *testing.T) {
	if test.CanRunTests() {
		Convey("StandingOrderUseCase", t, func() {
			useCase := rto.NewStandingOrderUseCase(repository, historyRepository)

			Convey("StandingOrder error "+rto.StandingOrderNotValidStateErrorText, func() {
				rtoModel := test.CreateRto(ctx, t, repository,
					api.RtoState_RTO_STATE_PENDING_SETUP, test.KorshilerDogs, test.Chmolgani)
				req := &request.StandingOrderRequest{
					ID: rtoModel.ID,
				}
				res, err := useCase.StandingOrder(ctx, req)
				So(res, ShouldBeNil)
				So(err, ShouldBeError)
				So(strings.Contains(err.Error(), rto.StandingOrderNotValidStateErrorText), ShouldBeTrue)

				rtoModel.State = api.RtoState_RTO_STATE_WAITING_FOR_STANDING_ORDER_SETUP
				err = repository.MustUpdate(ctx, rtoModel)
				So(err, ShouldBeNil)

				Convey("StandingOrder success", func() {
					res, err = useCase.StandingOrder(ctx, req)
					So(err, ShouldBeNil)
					So(res, ShouldNotBeNil)

					test.DeleteRto(ctx, t, repository, historyRepository, rtoModel.ID, 1)
				})
			})
		})
	}
}
