package rto_test

import (
	"strings"
	"testing"

	account "gitlab.mashplace.com/mashplace/backend/account/api"
	api "gitlab.mashplace.com/mashplace/backend/offer/api/rto"
	requestservice "gitlab.mashplace.com/mashplace/backend/offer/internal/app/domain/rto/resource/request/service"
	request "gitlab.mashplace.com/mashplace/backend/offer/internal/app/domain/rto/resource/request/usecase"
	mockappservice "gitlab.mashplace.com/mashplace/backend/offer/internal/app/service/mock"
	"gitlab.mashplace.com/mashplace/backend/offer/internal/app/usecase/rto"
	"gitlab.mashplace.com/mashplace/backend/offer/test"

	"github.com/golang/mock/gomock"
	. "github.com/smartystreets/goconvey/convey"
)

func TestUpdateLandlordUseCase(t *testing.T) {
	if test.CanRunTests() {
		Convey("UpdateLandlordUseCase", t, func() {
			ctrl := gomock.NewController(t)
			defer ctrl.Finish()

			accountService := mockappservice.NewMockAccountService(ctrl)
			useCase := rto.NewUpdateLandlordUseCase(repository, historyRepository, accountService)

			Convey("UpdateLandlordUseCase error "+rto.UpdateLandlordNotValidStateErrorText, func() {
				rtoModel := test.CreateRto(ctx, t, repository,
					api.RtoState_RTO_STATE_WAITING_FOR_STANDING_ORDER_SETUP, test.KorshilerDogs, test.Chmolgani)
				rtoID := rtoModel.ID.Hex()
				landlordID := "test landlord id"

				req := &request.UpdateLandlordRequest{
					ID:         rtoID,
					LandlordID: landlordID,
				}
				res, err := useCase.UpdateLandlord(ctx, req)
				So(res, ShouldBeNil)
				So(err, ShouldBeError)
				So(strings.Contains(err.Error(), rto.UpdateLandlordNotValidStateErrorText), ShouldBeTrue)

				Convey("UpdateLandlordUseCase error "+rto.UpdateLandlordCanNotChangeCurrentLandlordErrorText, func() {
					rtoModel.State = api.RtoState_RTO_STATE_PENDING_SETUP
					So(repository.MustUpdate(ctx, rtoModel), ShouldBeNil)

					res, err := useCase.UpdateLandlord(ctx, req)
					So(res, ShouldBeNil)
					So(err, ShouldBeError)
					So(strings.Contains(err.Error(), rto.UpdateLandlordCanNotChangeCurrentLandlordErrorText),
						ShouldBeTrue)

					Convey("UpdateLandlordUseCase success", func() {
						rtoModel.Landlord = nil
						So(repository.MustUpdate(ctx, rtoModel), ShouldBeNil)

						accountService.EXPECT().IsVeriffValid(ctx, &requestservice.IsVeriffValidRequest{
							LandlordID: landlordID,
						}).Return(nil)
						accountService.EXPECT().GetAccountByID(ctx, landlordID).Return(&account.Account{
							ViewedVideosIds: []int32{2065, 2032},
						}, nil)
						resp, err := useCase.UpdateLandlord(ctx, &request.UpdateLandlordRequest{
							ID:         rtoID,
							LandlordID: landlordID,
						})
						So(err, ShouldBeNil)
						So(resp, ShouldNotBeNil)

						test.DeleteRto(ctx, t, repository, historyRepository, resp.Rto.ID, 1)
					})
				})
			})
		})
	}
}
