package rto_test

import (
	"testing"

	api "gitlab.mashplace.com/mashplace/backend/offer/api/rto"
	requestservice "gitlab.mashplace.com/mashplace/backend/offer/internal/app/domain/rto/resource/request/service"
	request "gitlab.mashplace.com/mashplace/backend/offer/internal/app/domain/rto/resource/request/usecase"
	mockappservice "gitlab.mashplace.com/mashplace/backend/offer/internal/app/service/mock"
	"gitlab.mashplace.com/mashplace/backend/offer/internal/app/usecase/rto"
	"gitlab.mashplace.com/mashplace/backend/offer/test"

	"github.com/golang/mock/gomock"
	. "github.com/smartystreets/goconvey/convey"
)

func TestUpdateLeadTenantUseCase(t *testing.T) {
	if test.CanRunTests() {
		Convey("UpdateLeadTenantUse", t, func() {
			ctrl := gomock.NewController(t)
			defer ctrl.Finish()

			listingService := mockappservice.NewMockListingService(ctrl)
			useCase := rto.NewUpdateLeadTenantUseCase(repository, historyRepository, listingService)

			Convey("UpdateLeadTenantUse RTO_STATE_PENDING_SETUP", func() {
				rtoModel := test.CreateRto(ctx, t, repository, api.RtoState_RTO_STATE_PENDING_SETUP,
					test.KorshilerDogs, test.Chmolgani)
				rtoID := rtoModel.ID.Hex()
				leadTenantID := "korshilerchertichmoshiolsin"
				req := &request.UpdateLeadTenantRequest{
					LeadTenantID:    leadTenantID,
					LeadTenantEmail: test.ZhunusoviDogs,
				}
				res, err := useCase.UpdateLeadTenant(ctx, req)
				So(err, ShouldBeNil)
				So(len(res.IDs), ShouldBeZeroValue)
				So(len(res.PropertyIDs), ShouldBeZeroValue)

				rtoModel.LeadTenant.User = ""
				So(repository.MustUpdate(ctx, rtoModel), ShouldBeNil)

				res, err = useCase.UpdateLeadTenant(ctx, req)
				So(err, ShouldBeNil)
				So(len(res.IDs), ShouldEqual, 1)
				So(res.IDs[0], ShouldEqual, rtoID)

				rtoDB, err := repository.FindByID(ctx, rtoModel.ID)
				So(err, ShouldBeNil)
				So(rtoDB, ShouldNotBeNil)
				So(rtoDB.GetLeadTenantID(), ShouldEqual, leadTenantID)

				test.DeleteRto(ctx, t, repository, historyRepository, rtoModel.ID, 1)
			})

			Convey("UpdateLeadTenantUse RTO_STATE_ACTIVE", func() {
				leadTenantID := "kollegichertichmoshiolsin"
				rtoModel := test.CreateRto(ctx, t, repository, api.RtoState_RTO_STATE_ACTIVE,
					test.KorshilerDogs, test.Chmolgani)
				rtoID := rtoModel.ID.Hex()
				req := &request.UpdateLeadTenantRequest{
					LeadTenantID:    leadTenantID,
					LeadTenantEmail: test.ZhunusoviDogs,
				}
				res, err := useCase.UpdateLeadTenant(ctx, req)
				So(err, ShouldBeNil)
				So(len(res.IDs), ShouldBeZeroValue)
				So(len(res.PropertyIDs), ShouldBeZeroValue)

				rtoModel.LeadTenant.User = ""
				So(repository.MustUpdate(ctx, rtoModel), ShouldBeNil)

				listingService.EXPECT().UpdateTenantID(ctx, &requestservice.ListingUpdateTenantIDRequest{
					PropertyID:   rtoModel.Property,
					LeadTenantID: leadTenantID,
					LandlordID:   rtoModel.GetLandlordID(),
				}).Return(nil)
				res, err = useCase.UpdateLeadTenant(ctx, req)
				So(err, ShouldBeNil)
				So(len(res.IDs), ShouldEqual, 1)
				So(res.IDs[0], ShouldEqual, rtoID)
				So(res.PropertyIDs[0], ShouldEqual, rtoModel.Property)

				rtoDB, err := repository.FindByID(ctx, rtoModel.ID)
				So(err, ShouldBeNil)
				So(rtoDB, ShouldNotBeNil)
				So(rtoDB.GetLeadTenantID(), ShouldEqual, leadTenantID)

				test.DeleteRto(ctx, t, repository, historyRepository, rtoModel.ID, 1)
			})
		})
	}
}
