package rto_test

import (
	"strings"
	"testing"

	api "gitlab.mashplace.com/mashplace/backend/offer/api/rto"
	requestservrto "gitlab.mashplace.com/mashplace/backend/offer/internal/app/domain/rto/resource/request/service/rto"
	request "gitlab.mashplace.com/mashplace/backend/offer/internal/app/domain/rto/resource/request/usecase"
	"gitlab.mashplace.com/mashplace/backend/offer/internal/app/domain/rto/resource/response"
	mockservice "gitlab.mashplace.com/mashplace/backend/offer/internal/app/service/rto/mock"
	"gitlab.mashplace.com/mashplace/backend/offer/internal/app/usecase/rto"
	"gitlab.mashplace.com/mashplace/backend/offer/test"

	"github.com/golang/mock/gomock"
	. "github.com/smartystreets/goconvey/convey"
)

func TestUpdateRentPlanUseCase(t *testing.T) {
	if test.CanRunTests() {
		Convey("UpdateRentPlanUseCase", t, func() {
			ctrl := gomock.NewController(t)
			defer ctrl.Finish()

			securityService := mockservice.NewMockSecurityService(ctrl)
			useCase := rto.NewUpdateRentPlanUseCase(repository, historyRepository, securityService)

			Convey("UpdateRentPlan error "+rto.UpdateRentPlanNotValidStateErrorText, func() {
				rtoModel := test.CreateRto(ctx, t, repository,
					api.RtoState_RTO_STATE_WAITING_FOR_STANDING_ORDER_SETUP, test.KorshilerDogs, test.Chmolgani)
				rtoID, landlordID := rtoModel.ID.Hex(), rtoModel.Landlord.User

				securityService.EXPECT().CanLandlordUpdateRto(ctx, &requestservrto.CanLandlordUpdateRtoRequest{
					ID:         rtoID,
					LandlordID: landlordID,
				}).Return(&response.RtoResponse{Rto: rtoModel}, nil)
				res, err := useCase.UpdateRentPlan(ctx, &request.UpdateRentPlanRequest{
					ID:         rtoID,
					LandlordID: landlordID,
					RentPlan:   api.RentPlan_RENT_PLAN_ESSENTIAL,
				})
				So(res, ShouldBeNil)
				So(err, ShouldBeError)
				So(strings.Contains(err.Error(), rto.UpdateRentPlanNotValidStateErrorText), ShouldBeTrue)

				rtoModel.State = api.RtoState_RTO_STATE_PENDING_SETUP
				err = repository.MustUpdate(ctx, rtoModel)
				So(err, ShouldBeNil)

				Convey("UpdateRentPlan success", func() {
					securityService.EXPECT().CanLandlordUpdateRto(ctx, &requestservrto.CanLandlordUpdateRtoRequest{
						ID:         rtoID,
						LandlordID: landlordID,
					}).Return(&response.RtoResponse{Rto: rtoModel}, nil)
					res, err = useCase.UpdateRentPlan(ctx, &request.UpdateRentPlanRequest{
						ID:         rtoID,
						LandlordID: landlordID,
						RentPlan:   api.RentPlan_RENT_PLAN_ESSENTIAL,
					})
					So(err, ShouldBeNil)
					So(res, ShouldNotBeNil)

					test.DeleteRto(ctx, t, repository, historyRepository, rtoModel.ID, 1)
				})
			})
		})
	}
}
