package rto_test

import (
	"strings"
	"testing"

	api "gitlab.mashplace.com/mashplace/backend/offer/api/rto"
	requestservrto "gitlab.mashplace.com/mashplace/backend/offer/internal/app/domain/rto/resource/request/service/rto"
	request "gitlab.mashplace.com/mashplace/backend/offer/internal/app/domain/rto/resource/request/usecase"
	mockservicerto "gitlab.mashplace.com/mashplace/backend/offer/internal/app/service/rto/mock"
	"gitlab.mashplace.com/mashplace/backend/offer/internal/app/usecase/rto"
	"gitlab.mashplace.com/mashplace/backend/offer/test"

	"github.com/golang/mock/gomock"
	. "github.com/smartystreets/goconvey/convey"
)

func TestUpdateStateToActiveUseCase(t *testing.T) {
	if test.CanRunTests() {
		Convey("UpdateStateToActiveUseCase", t, func() {
			ctrl := gomock.NewController(t)
			defer ctrl.Finish()

			rtoNotificationService := mockservicerto.NewMockNotificationService(ctrl)
			useCase := rto.NewUpdateStateToActiveUseCase(repository,
				historyRepository, rtoNotificationService, closeComService)

			Convey("UpdateStateToActive error "+
				rto.UpdateStateToActiveNotValidStateErrorText, func() {
				rtoModel := test.CreateRto(ctx, t, repository, api.RtoState_RTO_STATE_PENDING_SETUP,
					test.KorshilerDogs, test.Chmolgani)
				rtoID := rtoModel.ID.Hex()
				saveOpportunityResponse, err := closeComService.SaveOpportunity(ctx,
					&requestservrto.CloseComSaveOpportunityRequest{
						RtoPreviousStateDate: rtoModel.UpdatedAt,
						Rto:                  rtoModel,
						RtoPreviousState:     rtoModel.State,
					})
				So(err, ShouldBeNil)
				So(saveOpportunityResponse.OpportunityID, ShouldNotBeEmpty)
				So(saveOpportunityResponse.LeadID, ShouldNotBeEmpty)

				rtoModel.OpportunityID = saveOpportunityResponse.OpportunityID
				rtoModel.Landlord.LeadID = saveOpportunityResponse.LeadID
				So(repository.MustUpdate(ctx, rtoModel), ShouldBeNil)

				req := &request.UpdateStateToActiveRequest{
					ID: rtoID,
				}
				res, err := useCase.UpdateStateToActive(ctx, req)
				So(res, ShouldBeNil)
				So(err, ShouldBeError)
				So(strings.Contains(err.Error(), rto.UpdateStateToActiveNotValidStateErrorText),
					ShouldBeTrue)

				rtoModel.State = api.RtoState_RTO_STATE_WAITING_FOR_STANDING_ORDER_SETUP
				err = repository.MustUpdate(ctx, rtoModel)
				So(err, ShouldBeNil)

				Convey("UpdateStateToActive success", func() {
					rtoNotificationService.EXPECT().SendActive(ctx, gomock.Any()).Return(nil)
					res, err := useCase.UpdateStateToActive(ctx, req)
					So(err, ShouldBeNil)
					So(res, ShouldNotBeNil)

					err = closeComService.DeleteOpportunity(ctx, saveOpportunityResponse.OpportunityID)
					So(err, ShouldBeNil)

					err = closeComService.DeleteLead(ctx, saveOpportunityResponse.LeadID)
					So(err, ShouldBeNil)

					test.DeleteRto(ctx, t, repository, historyRepository, rtoModel.ID, 1)
				})
			})
		})
	}
}
