package rto_test

import (
	"strings"
	"testing"
	"time"

	api "gitlab.mashplace.com/mashplace/backend/offer/api/rto"
	requestservrto "gitlab.mashplace.com/mashplace/backend/offer/internal/app/domain/rto/resource/request/service/rto"
	request "gitlab.mashplace.com/mashplace/backend/offer/internal/app/domain/rto/resource/request/usecase"
	"gitlab.mashplace.com/mashplace/backend/offer/internal/app/usecase/rto"
	"gitlab.mashplace.com/mashplace/backend/offer/test"

	. "github.com/smartystreets/goconvey/convey"
)

func TestUpdateStateToCancelledUseCase(t *testing.T) {
	if test.CanRunTests() {
		Convey("UpdateStateToCancelledUseCase", t, func() {
			useCase := rto.NewUpdateStateToCancelledUseCase(repository, historyRepository, closeComService)

			Convey("UpdateStateToCancelled error "+
				rto.UpdateStateToCancelledNotValidStateErrorText, func() {
				rtoModel := test.CreateRto(ctx, t, repository, api.RtoState_RTO_STATE_PENDING_SETUP,
					test.KorshilerDogs, test.Chmolgani)
				saveOpportunityResponse, err := closeComService.SaveOpportunity(ctx,
					&requestservrto.CloseComSaveOpportunityRequest{
						RtoPreviousStateDate: rtoModel.UpdatedAt,
						Rto:                  rtoModel,
						RtoPreviousState:     rtoModel.State,
					})
				So(err, ShouldBeNil)
				So(saveOpportunityResponse.OpportunityID, ShouldNotBeEmpty)
				So(saveOpportunityResponse.LeadID, ShouldNotBeEmpty)

				rtoModel.OpportunityID = saveOpportunityResponse.OpportunityID
				rtoModel.Landlord.LeadID = saveOpportunityResponse.LeadID
				So(repository.MustUpdate(ctx, rtoModel), ShouldBeNil)

				rtoID := rtoModel.ID.Hex()
				req := &request.UpdateStateToCancelledRequest{
					ID: rtoID,
				}
				res, err := useCase.UpdateStateToCancelled(ctx, req)
				So(res, ShouldBeNil)
				So(err, ShouldBeError)
				So(strings.Contains(err.Error(), rto.UpdateStateToCancelledNotValidStateErrorText), ShouldBeTrue)

				Convey("UpdateStateToCancelled error "+
					rto.UpdateStateToCancelledNotValidRtoEndDateErrorText, func() {
					rtoModel.State = api.RtoState_RTO_STATE_ENDING
					rtoModel.Conditions.EndDate = time.Now().Add(59.5 * 24 * time.Hour)
					err = repository.MustUpdate(ctx, rtoModel)
					So(err, ShouldBeNil)

					res, err = useCase.UpdateStateToCancelled(ctx, req)
					So(res, ShouldBeNil)
					So(err, ShouldBeError)
					So(strings.Contains(err.Error(),
						rto.UpdateStateToCancelledNotValidRtoEndDateErrorText), ShouldBeTrue)

					rtoModel.Conditions.EndDate = time.Now().Add(60.5 * 24 * time.Hour)
					err = repository.MustUpdate(ctx, rtoModel)
					So(err, ShouldBeNil)

					Convey("UpdateStateToCancelled success", func() {
						res, err := useCase.UpdateStateToCancelled(ctx, req)
						So(err, ShouldBeNil)
						So(res, ShouldNotBeNil)

						err = closeComService.DeleteOpportunity(ctx, saveOpportunityResponse.OpportunityID)
						So(err, ShouldBeNil)

						err = closeComService.DeleteLead(ctx, saveOpportunityResponse.LeadID)
						So(err, ShouldBeNil)

						test.DeleteRto(ctx, t, repository, historyRepository, rtoModel.ID, 1)
					})
				})
			})
		})
	}
}
