package rto_test

import (
	"strings"
	"testing"

	api "gitlab.mashplace.com/mashplace/backend/offer/api/rto"
	requestservrto "gitlab.mashplace.com/mashplace/backend/offer/internal/app/domain/rto/resource/request/service/rto"
	request "gitlab.mashplace.com/mashplace/backend/offer/internal/app/domain/rto/resource/request/usecase"
	"gitlab.mashplace.com/mashplace/backend/offer/internal/app/usecase/rto"
	"gitlab.mashplace.com/mashplace/backend/offer/test"

	. "github.com/smartystreets/goconvey/convey"
)

func TestUpdateStateToFinishedUseCase(t *testing.T) {
	if test.CanRunTests() {
		Convey("UpdateStateToFinishedUseCase", t, func() {
			useCase := rto.NewUpdateStateToFinishedUseCase(repository, historyRepository, closeComService)

			Convey("UpdateStateToFinished error "+
				rto.UpdateStateToFinishedNotValidStateErrorText, func() {
				rtoModel := test.CreateRto(ctx, t, repository, api.RtoState_RTO_STATE_PENDING_SETUP,
					test.KorshilerDogs, test.Chmolgani)
				rtoID := rtoModel.ID.Hex()
				saveOpportunityResponse, err := closeComService.SaveOpportunity(ctx,
					&requestservrto.CloseComSaveOpportunityRequest{
						RtoPreviousStateDate: rtoModel.UpdatedAt,
						Rto:                  rtoModel,
						RtoPreviousState:     rtoModel.State,
					})
				So(err, ShouldBeNil)
				So(saveOpportunityResponse.OpportunityID, ShouldNotBeEmpty)
				So(saveOpportunityResponse.LeadID, ShouldNotBeEmpty)

				rtoModel.OpportunityID = saveOpportunityResponse.OpportunityID
				rtoModel.Landlord.LeadID = saveOpportunityResponse.LeadID
				So(repository.MustUpdate(ctx, rtoModel), ShouldBeNil)

				req := &request.UpdateStateToFinishedRequest{
					ID: rtoID,
				}
				res, err := useCase.UpdateStateToFinished(ctx, req)
				So(res, ShouldBeNil)
				So(err, ShouldBeError)
				So(strings.Contains(err.Error(), rto.UpdateStateToFinishedNotValidStateErrorText),
					ShouldBeTrue)

				rtoModel.State = api.RtoState_RTO_STATE_ENDING
				err = repository.MustUpdate(ctx, rtoModel)
				So(err, ShouldBeNil)

				Convey("UpdateStateToFinished success", func() {
					res, err := useCase.UpdateStateToFinished(ctx, req)
					So(err, ShouldBeNil)
					So(res, ShouldNotBeNil)

					err = closeComService.DeleteOpportunity(ctx, saveOpportunityResponse.OpportunityID)
					So(err, ShouldBeNil)

					err = closeComService.DeleteLead(ctx, saveOpportunityResponse.LeadID)
					So(err, ShouldBeNil)

					test.DeleteRto(ctx, t, repository, historyRepository, rtoModel.ID, 1)
				})
			})
		})
	}
}
