package rto_test

import (
	"strings"
	"testing"

	api "gitlab.mashplace.com/mashplace/backend/offer/api/rto"
	requestservice "gitlab.mashplace.com/mashplace/backend/offer/internal/app/domain/rto/resource/request/service"
	requestservrto "gitlab.mashplace.com/mashplace/backend/offer/internal/app/domain/rto/resource/request/service/rto"
	request "gitlab.mashplace.com/mashplace/backend/offer/internal/app/domain/rto/resource/request/usecase"
	mockappservice "gitlab.mashplace.com/mashplace/backend/offer/internal/app/service/mock"
	mockservicerto "gitlab.mashplace.com/mashplace/backend/offer/internal/app/service/rto/mock"
	"gitlab.mashplace.com/mashplace/backend/offer/internal/app/usecase/rto"
	"gitlab.mashplace.com/mashplace/backend/offer/test"

	"github.com/golang/mock/gomock"
	. "github.com/smartystreets/goconvey/convey"
)

func TestUpdateStateToWaitingForTenantConfirmationUseCase(t *testing.T) {
	if test.CanRunTests() {
		Convey("UpdateStateToWaitingForTenantConfirmationUseCase", t, func() {
			ctrl := gomock.NewController(t)
			defer ctrl.Finish()

			accountService := mockappservice.NewMockAccountService(ctrl)
			rtoNotificationService := mockservicerto.NewMockNotificationService(ctrl)
			listingService := mockappservice.NewMockListingService(ctrl)
			updateLeadTenantUserUseCase := rto.NewUpdateLeadTenantUserUseCase(
				repository, historyRepository, accountService, listingService)
			useCase := rto.NewUpdateStateToWaitingForTenantConfirmationUseCase(repository, historyRepository,
				updateLeadTenantUserUseCase, rtoNotificationService, closeComService, listingService)

			Convey("UpdateStateToWaitingForTenantConfirmation error "+
				rto.UpdateStateToWaitingForTenantConfirmationNotValidStateErrorText, func() {
				rtoModel := test.CreateRto(ctx, t, repository, api.RtoState_RTO_STATE_WAITING_FOR_STANDING_ORDER_SETUP,
					test.KorshilerDogs, test.Chmolgani)
				saveOpportunityResponse, err := closeComService.SaveOpportunity(ctx,
					&requestservrto.CloseComSaveOpportunityRequest{
						RtoPreviousStateDate: rtoModel.UpdatedAt,
						Rto:                  rtoModel,
						RtoPreviousState:     rtoModel.State,
					})
				So(err, ShouldBeNil)
				So(saveOpportunityResponse.OpportunityID, ShouldNotBeEmpty)
				So(saveOpportunityResponse.LeadID, ShouldNotBeEmpty)

				req := &request.UpdateStateToWaitingForTenantConfirmationRequest{
					Rto: rtoModel,
				}
				res, err := useCase.UpdateStateToWaitingForTenantConfirmation(ctx, req)
				So(res, ShouldBeNil)
				So(err, ShouldBeError)
				So(strings.Contains(err.Error(), rto.UpdateStateToWaitingForTenantConfirmationNotValidStateErrorText),
					ShouldBeTrue)

				rtoModel.State = api.RtoState_RTO_STATE_PENDING_SETUP
				err = repository.MustUpdate(ctx, rtoModel)
				So(err, ShouldBeNil)

				Convey("UpdateStateToWaitingForTenantConfirmation success", func() {
					rtoNotificationService.EXPECT().SendWaitingForStandingOrderSetup(ctx, gomock.Any()).Return(nil)
					listingService.EXPECT().UpdateTenantID(ctx, &requestservice.ListingUpdateTenantIDRequest{
						PropertyID:   rtoModel.Property,
						LeadTenantID: rtoModel.GetLeadTenantID(),
						LandlordID:   rtoModel.GetLandlordID(),
					}).Return(nil)
					res, err := useCase.UpdateStateToWaitingForTenantConfirmation(ctx, req)
					So(err, ShouldBeNil)
					So(res, ShouldNotBeNil)

					err = closeComService.DeleteOpportunity(ctx, saveOpportunityResponse.OpportunityID)
					So(err, ShouldBeNil)

					err = closeComService.DeleteLead(ctx, saveOpportunityResponse.LeadID)
					So(err, ShouldBeNil)

					test.DeleteRto(ctx, t, repository, historyRepository, rtoModel.ID, 1)
				})
			})
		})
	}
}
