// Package invoices is intended to keep in one place parameters used to access invoice-service.
package invoices

import (
	"context"

	invoice "gitlab.mashplace.com/mashplace/backend/invoice/api"
	"gitlab.mashplace.com/mashplace/backend/offer/internal/connections"
)

// HoldingDepositPayment checks specified payment is for holding deposit
func HoldingDepositPayment(p *invoice.Payment) bool {
	return p != nil &&
		p.CorrelationType == invoice.CorrelationTypeOffer &&
		p.CorrelationTarget == invoice.CorrelationTargetHoldingDeposit
}

// HoldingDepositHasPaid finds "holding deposit" invoices for given offer User
// and checks there was one successfully paid.
func HoldingDepositHasPaid(ctx context.Context, offerID string) (bool, error) {
	res, err := connections.InvoiceService.InvoiceList(ctx, &invoice.InvoiceListRequest{
		CorrelationType:   invoice.CorrelationTypeOffer,
		CorrelationId:     offerID,
		CorrelationTarget: invoice.CorrelationTargetHoldingDeposit,
		Type:              invoice.InvoiceType_INVOICE_TYPE_PAYMENT,
		Statuses: []invoice.InvoiceStatus{
			invoice.InvoiceStatus_INVOICE_FULFILLED,
		},
	})

	if err != nil {
		return false, err
	}

	return len(res.GetInvoices()) > 0, nil
}
