package status

import (
	"context"
	"time"

	invoice "gitlab.mashplace.com/mashplace/backend/invoice/api"

	"gitlab.mashplace.com/mashplace/backend/common/api/grpcctx"
	listing "gitlab.mashplace.com/mashplace/backend/listing/api"

	extras "gitlab.mashplace.com/mashplace/backend/extras/api"

	closecom "gitlab.mashplace.com/mashplace/backend/closecom/api"
	"gitlab.mashplace.com/mashplace/backend/common/generic/log"
	"gitlab.mashplace.com/mashplace/backend/common/generic/util"
	"gitlab.mashplace.com/mashplace/backend/offer/internal/connections"
	"gitlab.mashplace.com/mashplace/backend/offer/model"
)

// ExportClosecomOpportunity exports offer status transition to closecom serrvice
func ExportClosecomOpportunity(
	ctx context.Context,
	m *model.OfferModel,
	status closecom.OfferStatus,
) {
	if m.OpportunityTerminated {
		return
	}

	changet := m.StateChangedAt
	if changet.IsZero() {
		changet = time.Now()
	}

	var termMonths int32

	var rentAmount, drpAmount, referencingAmount float64

	var moveint time.Time

	renewal := m.OriginalOffer != ""

	drp := m.DepositReplacement()

	if m.Conditions != nil {
		termMonths = util.UnPtrInt32(m.Conditions.TermMonths)
		rentAmount = util.UnPtrFloat64(m.Conditions.Price)

		if drp && !renewal {
			price := extras.CalculateDepositReplacementAmount(rentAmount)

			drpAmount = price

			service, _ := extras.GetServicePriceSnapshot(
				ctx,
				connections.ExtrasService,
				extras.ProductTenantDepositReplacement,
				drpAmount,
			)

			drpAmount += service.GetTaxAmount()
		}

		moveint = util.UnPtrTime(m.Conditions.MoveinDate)
	}

	if !renewal {
		invoiceList, _ := connections.InvoiceService.InvoiceList(ctx, &invoice.InvoiceListRequest{
			CorrelationType:   invoice.CorrelationTypeOffer,
			CorrelationId:     m.ID.Hex(),
			CorrelationTarget: invoice.CorrelationTargetReferenceCheck,
			Statuses:          []invoice.InvoiceStatus{invoice.InvoiceStatus_INVOICE_FULFILLED},
		})

		for _, v := range invoiceList.GetInvoices() {
			referencingAmount += v.GetAmount()
		}
	}

	if m.PropertyAddress == "" {
		ver, _ := connections.ListingService.VersionGet(
			grpcctx.WithServiceUser(ctx),
			&listing.VersionGetRequest{
				Spec: listing.VersionSpecPropertyID("", m.Property),
			},
		)

		m.PropertyAddress = ver.GetVersion().GetLocation().FormatAddress()
	}

	resp, err := connections.ClosecomService.LeadOpportunitySave(ctx, &closecom.LeadOpportunitySaveRequest{
		UserId: m.LandlordUser(),
		Opportunity: &closecom.LeadOpportunitySaveRequest_Offer{
			Offer: &closecom.Offer{
				OpportunityId:     m.OpportunityID,
				OfferId:           m.ID.Hex(),
				PropertyId:        m.Property,
				TenantUserId:      m.TenantUser(),
				TermMonths:        termMonths,
				RentAmount:        rentAmount,
				DrpAmount:         drpAmount,
				ReferencingAmount: referencingAmount,
				MoveinDate:        util.ToDate(moveint),
				TerminationDate:   util.ToDate(moveint.AddDate(0, int(termMonths), 0)),
				StatusChangeDate:  util.ToDate(changet),
				Status:            status,
				Renewal:           renewal,
				PropertyAddress:   m.PropertyAddress,
			},
		},
	})
	if err != nil {
		log.WithContext(ctx).WithError(err).WithField("offer", m.ID.Hex()).Error("exporting closecom opportunity")

		return
	}

	m.OpportunityID = resp.GetOpportunityId()
	m.OpportunityTerminated = resp.GetOpportunityTerminated()
}
