package synchronization

import (
	"context"

	offer "gitlab.mashplace.com/mashplace/backend/offer/api"
	"go.mongodb.org/mongo-driver/bson/primitive"

	"gitlab.mashplace.com/mashplace/backend/common/api/grpcctx"
	"gitlab.mashplace.com/mashplace/backend/common/generic/log"
	listing "gitlab.mashplace.com/mashplace/backend/listing/api"
	"gitlab.mashplace.com/mashplace/backend/offer/internal/connections"
	"gitlab.mashplace.com/mashplace/backend/offer/model"
)

// ServiceOfferSync sync all external dependencies on offer
func ServiceOfferSync(ctx context.Context, m *model.OfferModel) {
	bgctx := grpcctx.ValuesBackgroundContext(ctx)

	go ServiceOfferSyncListing(bgctx, m.ID, &m.Sync, m.Property, m.PropertyListing, m.TenantUser(), m.State)
}

// ServiceOfferSyncListing sync listing external dependency on offer
func ServiceOfferSyncListing(
	ctx context.Context,
	id primitive.ObjectID,
	sync *model.OfferSync,
	propertyID, listingID, tenant string,
	state offer.OfferState,
) {
	if sync.Listing {
		return
	}

	if tenant == "" {
		return
	}

	var spec *listing.VersionSpec

	if listingID != "" {
		spec = listing.VersionSpecListingID("", listingID)
	} else {
		spec = listing.VersionSpecPropertyID("", propertyID)
	}

	resp, err := connections.ListingService.ServiceVersionRelatedResource(
		ctx,
		&listing.ServiceVersionRelatedResourceRequest{
			Spec:     spec,
			UserId:   tenant,
			State:    int32(state),
			Resource: listing.RelatedResource_RELATED_RESOURCE_OFFER,
		},
	)
	if err != nil {
		log.WithError(err).WithField("offer_id", id).
			Error("syncing listing offer resource")

		return
	}

	err = model.OfferSetSyncFlag(ctx, id, model.SyncKindListing, state)
	if err != nil {
		log.WithError(err).WithField("offer_id", id.Hex()).
			Error("saving offer sync status")

		return
	}

	if listingID == "" {
		err = model.OfferSetListingID(ctx, id, resp.GetListingId())
		if err != nil {
			log.WithError(err).WithField("offer_id", id.Hex()).
				Error("updating offer listing id")
		}
	}
}
