package model

import (
	"context"

	offer "gitlab.mashplace.com/mashplace/backend/offer/api"
	"go.mongodb.org/mongo-driver/bson"
	"go.mongodb.org/mongo-driver/bson/primitive"
)

// SyncKind synchronization kind
type SyncKind string

const (
	// SyncKindListing listing synchronization
	SyncKindListing SyncKind = "listing"
)

// OfferSetSyncFlag marks offer as synced for this kind of synchronization
func OfferSetSyncFlag(ctx context.Context, id primitive.ObjectID, kind SyncKind, status offer.OfferState) error {
	_, err := Offers.UpdateOne(
		ctx,
		bson.M{"_id": id, "status": status},
		bson.A{
			bson.M{
				"$set": bson.M{
					"sync": bson.M{
						"$cond": bson.M{
							"if":   bson.M{"$not": bson.M{"sync": bson.M{"$type": "object"}}},
							"then": bson.M{string(kind): true},
							"else": bson.M{"$mergeObjects": bson.A{"$sync", bson.M{string(kind): true}}},
						},
					},
				},
			},
		},
	)

	return err
}

// OfferSetListingID sets listing ID
func OfferSetListingID(ctx context.Context, id primitive.ObjectID, listingID string) error {
	_, err := Offers.UpdateOne(
		ctx,
		bson.M{"_id": id},
		bson.M{"$set": bson.M{"property_listing": listingID}},
	)

	return err
}

// OfferUnsynchronizedModel reduced offer model for synchronization
type OfferUnsynchronizedModel struct {
	Property        string              `bson:"property"`
	PropertyListing string              `bson:"property_listing"`
	Tenants         []*TenantOfferModel `bson:"tenants"`
	State           offer.OfferState    `bson:"state"`
	ID              primitive.ObjectID  `bson:"_id"`
	Sync            OfferSync           `bson:"sync"`
}

// OfferUnsynchronizedList sets listing ID
func OfferUnsynchronizedList(ctx context.Context) (ms []OfferUnsynchronizedModel, err error) {
	cur, err := Offers.Find(ctx, bson.M{
		"$or": bson.A{
			bson.M{"sync.listing": bson.M{"$ne": true}},
		},
	})
	if err != nil {
		return
	}

	err = cur.All(ctx, &ms)
	if err != nil {
		return
	}

	return
}
